@@ -42,13 +42,12 @@
 #include <logging/translator.h>
 #include <tools/error.h>
 #include <tools/installoptions.h>
+#include <tools/qttools.h>
 
 namespace qbs {
 using namespace Internal;
 
-CommandLineOption::~CommandLineOption()
-{
-}
+CommandLineOption::~CommandLineOption() = default;
 
 void CommandLineOption::parse(CommandType command, const QString &representation, QStringList &input)
 {
@@ -63,7 +62,7 @@ CommandLineOption::CommandLineOption()
 
 QString CommandLineOption::getArgument(const QString &representation, QStringList &input)
 {
-    if (input.isEmpty()) {
+    if (input.empty()) {
         throw ErrorInfo(Tr::tr("Invalid use of option '%1': Missing argument.\nUsage: %2")
                     .arg(representation, description(command())));
     }
@@ -83,12 +82,12 @@ QString FileOption::description(CommandType command) const
 
 QString FileOption::shortRepresentation() const
 {
-    return QLatin1String("-f");
+    return QStringLiteral("-f");
 }
 
 QString FileOption::longRepresentation() const
 {
-    return QLatin1String("--file");
+    return QStringLiteral("--file");
 }
 
 void FileOption::doParse(const QString &representation, QStringList &input)
@@ -113,22 +112,22 @@ QString BuildDirectoryOption::description(CommandType command) const
 
 QString BuildDirectoryOption::shortRepresentation() const
 {
-    return QLatin1String("-d");
+    return QStringLiteral("-d");
 }
 
 QString BuildDirectoryOption::longRepresentation() const
 {
-    return QLatin1String("--build-directory");
+    return QStringLiteral("--build-directory");
 }
 
 QString BuildDirectoryOption::magicProjectString()
 {
-    return QLatin1String("@project");
+    return QStringLiteral("@project");
 }
 
 QString BuildDirectoryOption::magicProjectDirString()
 {
-    return QLatin1String("@path");
+    return QStringLiteral("@path");
 }
 
 void BuildDirectoryOption::doParse(const QString &representation, QStringList &input)
@@ -146,12 +145,12 @@ QString GeneratorOption::description(CommandType command) const
 
 QString GeneratorOption::shortRepresentation() const
 {
-    return QLatin1String("-g");
+    return QStringLiteral("-g");
 }
 
 QString GeneratorOption::longRepresentation() const
 {
-    return QLatin1String("--generator");
+    return QStringLiteral("--generator");
 }
 
 void GeneratorOption::doParse(const QString &representation, QStringList &input)
@@ -163,7 +162,7 @@ void GeneratorOption::doParse(const QString &representation, QStringList &input)
     }
 }
 
-static QString loglevelLongRepresentation() { return QLatin1String("--log-level"); }
+static QString loglevelLongRepresentation() { return QStringLiteral("--log-level"); }
 
 QString VerboseOption::description(CommandType command) const
 {
@@ -179,12 +178,12 @@ QString VerboseOption::description(CommandType command) const
 
 QString VerboseOption::shortRepresentation() const
 {
-    return QLatin1String("-v");
+    return QStringLiteral("-v");
 }
 
 QString VerboseOption::longRepresentation() const
 {
-    return QLatin1String("--more-verbose");
+    return QStringLiteral("--more-verbose");
 }
 
 QString QuietOption::description(CommandType command) const
@@ -201,12 +200,12 @@ QString QuietOption::description(CommandType command) const
 
 QString QuietOption::shortRepresentation() const
 {
-    return QLatin1String("-q");
+    return QStringLiteral("-q");
 }
 
 QString QuietOption::longRepresentation() const
 {
-    return QLatin1String("--less-verbose");
+    return QStringLiteral("--less-verbose");
 }
 
 QString JobsOption::description(CommandType command) const
@@ -220,12 +219,12 @@ QString JobsOption::description(CommandType command) const
 
 QString JobsOption::shortRepresentation() const
 {
-    return QLatin1String("-j");
+    return QStringLiteral("-j");
 }
 
 QString JobsOption::longRepresentation() const
 {
-    return QLatin1String("--jobs");
+    return QStringLiteral("--jobs");
 }
 
 void JobsOption::doParse(const QString &representation, QStringList &input)
@@ -248,12 +247,12 @@ QString KeepGoingOption::description(CommandType command) const
 
 QString KeepGoingOption::shortRepresentation() const
 {
-    return QLatin1String("-k");
+    return QStringLiteral("-k");
 }
 
 QString KeepGoingOption::longRepresentation() const
 {
-    return QLatin1String("--keep-going");
+    return QStringLiteral("--keep-going");
 }
 
 QString DryRunOption::description(CommandType command) const
@@ -267,12 +266,12 @@ QString DryRunOption::description(CommandType command) const
 
 QString DryRunOption::shortRepresentation() const
 {
-    return QLatin1String("-n");
+    return QStringLiteral("-n");
 }
 
 QString DryRunOption::longRepresentation() const
 {
-    return QLatin1String("--dry-run");
+    return QStringLiteral("--dry-run");
 }
 
 QString ForceProbesOption::description(CommandType command) const
@@ -286,7 +285,7 @@ QString ForceProbesOption::description(CommandType command) const
 
 QString ForceProbesOption::longRepresentation() const
 {
-    return QLatin1String("--force-probe-execution");
+    return QStringLiteral("--force-probe-execution");
 }
 
 QString NoInstallOption::description(CommandType command) const
@@ -299,13 +298,13 @@ QString NoInstallOption::description(CommandType command) const
 
 QString NoInstallOption::longRepresentation() const
 {
-    return QLatin1String("--no-install");
+    return QStringLiteral("--no-install");
 }
 
 
 static QString logTimeRepresentation()
 {
-    return QLatin1String("--log-time");
+    return QStringLiteral("--log-time");
 }
 
 QString ShowProgressOption::description(CommandType command) const
@@ -320,7 +319,7 @@ QString ShowProgressOption::description(CommandType command) const
 
 static QString showProgressRepresentation()
 {
-    return QLatin1String("--show-progress");
+    return QStringLiteral("--show-progress");
 }
 
 QString ShowProgressOption::longRepresentation() const
@@ -331,11 +330,11 @@ QString ShowProgressOption::longRepresentation() const
 void StringListOption::doParse(const QString &representation, QStringList &input)
 {
     m_arguments = getArgument(representation, input).split(QLatin1Char(','));
-    if (m_arguments.isEmpty()) {
+    if (m_arguments.empty()) {
         throw ErrorInfo(Tr::tr("Invalid use of option '%1': Argument list must not be empty.\n"
                            "Usage: %2").arg(representation, description(command())));
     }
-    foreach (const QString &element, m_arguments) {
+    for (const QString &element : std::as_const(m_arguments)) {
         if (element.isEmpty()) {
             throw ErrorInfo(Tr::tr("Invalid use of option '%1': Argument list must not contain "
                                "empty elements.\nUsage: %2")
@@ -353,7 +352,7 @@ QString ChangedFilesOption::description(CommandType command) const
 
 QString ChangedFilesOption::longRepresentation() const
 {
-    return QLatin1String("--changed-files");
+    return QStringLiteral("--changed-files");
 }
 
 QString ProductsOption::description(CommandType command) const
@@ -372,12 +371,12 @@ QString ProductsOption::description(CommandType command) const
 
 QString ProductsOption::shortRepresentation() const
 {
-    return QLatin1String("-p");
+    return QStringLiteral("-p");
 }
 
 QString ProductsOption::longRepresentation() const
 {
-    return QLatin1String("--products");
+    return QStringLiteral("--products");
 }
 
 static QStringList allLogLevelStrings()
@@ -412,7 +411,7 @@ void LogLevelOption::doParse(const QString &representation, QStringList &input)
     const QString levelString = getArgument(representation, input);
     const QList<LoggerLevel> levels = QList<LoggerLevel>() << LoggerError << LoggerWarning
             << LoggerInfo << LoggerDebug << LoggerTrace;
-    foreach (LoggerLevel l, levels) {
+    for (const LoggerLevel &l : levels) {
         if (logLevelName(l) == levelString) {
             m_logLevel = l;
             return;
@@ -432,7 +431,7 @@ QString ForceTimeStampCheckOption::description(CommandType command) const
 
 QString ForceTimeStampCheckOption::longRepresentation() const
 {
-    return QLatin1String("--check-timestamps");
+    return QStringLiteral("--check-timestamps");
 }
 
 QString ForceOutputCheckOption::description(CommandType command) const
@@ -445,7 +444,7 @@ QString ForceOutputCheckOption::description(CommandType command) const
 
 QString ForceOutputCheckOption::longRepresentation() const
 {
-    return QLatin1String("--check-outputs");
+    return QStringLiteral("--check-outputs");
 }
 
 QString BuildNonDefaultOption::description(CommandType command) const
@@ -457,24 +456,7 @@ QString BuildNonDefaultOption::description(CommandType command) const
 
 QString BuildNonDefaultOption::longRepresentation() const
 {
-    return QLatin1String("--all-products");
-}
-
-QString VersionOption::description(CommandType command) const
-{
-    Q_UNUSED(command);
-    return Tr::tr("%1\n"
-            "\tDisplay the qbs version and exit.\n").arg(longRepresentation());
-}
-
-QString VersionOption::shortRepresentation() const
-{
-    return QString();
-}
-
-QString VersionOption::longRepresentation() const
-{
-    return QStringLiteral("--version");
+    return QStringLiteral("--all-products");
 }
 
 
@@ -482,7 +464,7 @@ InstallRootOption::InstallRootOption() : m_useSysroot(false)
 {
 }
 
-static QString magicSysrootString() { return QLatin1String("@sysroot"); }
+static QString magicSysrootString() { return QStringLiteral("@sysroot"); }
 
 QString InstallRootOption::description(CommandType command) const
 {
@@ -498,12 +480,12 @@ QString InstallRootOption::description(CommandType command) const
 
 QString InstallRootOption::longRepresentation() const
 {
-    return QLatin1String("--install-root");
+    return QStringLiteral("--install-root");
 }
 
 void InstallRootOption::doParse(const QString &representation, QStringList &input)
 {
-    if (input.isEmpty()) {
+    if (input.empty()) {
         throw ErrorInfo(Tr::tr("Invalid use of option '%1: Argument expected.\n"
                            "Usage: %2").arg(representation, description(command())));
     }
@@ -523,7 +505,7 @@ QString RemoveFirstOption::description(CommandType command) const
 
 QString RemoveFirstOption::longRepresentation() const
 {
-    return QLatin1String("--clean-install-root");
+    return QStringLiteral("--clean-install-root");
 }
 
 
@@ -536,7 +518,7 @@ QString NoBuildOption::description(CommandType command) const
 
 QString NoBuildOption::longRepresentation() const
 {
-    return QLatin1String("--no-build");
+    return QStringLiteral("--no-build");
 }
 
 
@@ -553,7 +535,7 @@ QString LogTimeOption::description(CommandType command) const
 
 QString LogTimeOption::shortRepresentation() const
 {
-    return QLatin1String("-t");
+    return QStringLiteral("-t");
 }
 
 QString LogTimeOption::longRepresentation() const
@@ -562,9 +544,7 @@ QString LogTimeOption::longRepresentation() const
 }
 
 
-SettingsDirOption::SettingsDirOption()
-{
-}
+SettingsDirOption::SettingsDirOption() = default;
 
 QString SettingsDirOption::description(CommandType command) const
 {
@@ -578,22 +558,73 @@ QString SettingsDirOption::description(CommandType command) const
 
 QString SettingsDirOption::longRepresentation() const
 {
-    return QLatin1String("--settings-dir");
+    return QStringLiteral("--settings-dir");
 }
 
 void SettingsDirOption::doParse(const QString &representation, QStringList &input)
 {
-    if (input.isEmpty()) {
+    if (input.empty()) {
         throw ErrorInfo(Tr::tr("Invalid use of option '%1: Argument expected.\n"
                            "Usage: %2").arg(representation, description(command())));
     }
     m_settingsDir = input.takeFirst();
 }
 
-CommandEchoModeOption::CommandEchoModeOption()
+QString JobLimitsOption::description(CommandType command) const
 {
+    Q_UNUSED(command);
+    return Tr::tr("%1 <pool1>:<limit1>[,<pool2>:<limit2>...]\n"
+                  "\tSet pool-specific job limits.\n").arg(longRepresentation());
 }
 
+QString JobLimitsOption::longRepresentation() const
+{
+    return QStringLiteral("--job-limits");
+}
+
+void JobLimitsOption::doParse(const QString &representation, QStringList &input)
+{
+    if (input.empty()) {
+        throw ErrorInfo(Tr::tr("Invalid use of option '%1: Argument expected.\n"
+                           "Usage: %2").arg(representation, description(command())));
+    }
+    const QString jobLimitsSpec = input.takeFirst();
+    const QStringList jobLimitStrings = jobLimitsSpec.split(QLatin1Char(','));
+    for (const QString &jobLimitString : jobLimitStrings) {
+        const int sepIndex = jobLimitString.indexOf(QLatin1Char(':'));
+        if (sepIndex <= 0 || sepIndex == jobLimitString.size() - 1) {
+            throw ErrorInfo(Tr::tr("Invalid use of option '%1: "
+                                   "Invalid job limits specification '%2'.\n"
+                                   "Usage: %3").arg(representation, jobLimitsSpec,
+                                                    description(command())));
+        }
+        const QString pool = jobLimitString.left(sepIndex);
+        const QString limitString = jobLimitString.mid(sepIndex + 1);
+        bool isValidNumber;
+        const int limit = limitString.toInt(&isValidNumber);
+        if (!isValidNumber) {
+            throw ErrorInfo(Tr::tr("Invalid use of option '%1: '%2' is not a number.\n"
+                                   "Usage: %3").arg(representation, limitString,
+                                                    description(command())));
+        }
+        m_jobLimits.setJobLimit(pool, limit);
+    }
+}
+
+QString RespectProjectJobLimitsOption::description(CommandType command) const
+{
+    Q_UNUSED(command);
+    return Tr::tr("%1\n\tGive maximum priority to job limits defined inside the project.\n")
+            .arg(longRepresentation());
+}
+
+QString RespectProjectJobLimitsOption::longRepresentation() const
+{
+    return QStringLiteral("--enforce-project-job-limits");
+}
+
+CommandEchoModeOption::CommandEchoModeOption() = default;
+
 QString CommandEchoModeOption::description(CommandType command) const
 {
     Q_UNUSED(command);
@@ -607,7 +638,7 @@ QString CommandEchoModeOption::description(CommandType command) const
 
 QString CommandEchoModeOption::longRepresentation() const
 {
-    return QLatin1String("--command-echo-mode");
+    return QStringLiteral("--command-echo-mode");
 }
 
 CommandEchoMode CommandEchoModeOption::commandEchoMode() const
@@ -632,6 +663,41 @@ void CommandEchoModeOption::doParse(const QString &representation, QStringList &
     m_echoMode = commandEchoModeFromName(mode);
 }
 
+QString DeprecationWarningsOption::description(CommandType command) const
+{
+    Q_UNUSED(command);
+    return Tr::tr("%1 <mode>\n"
+                  "\tWhat to do when encountering deprecated items or properties.\n"
+                  "\tPossible values are '%2'.\n"
+                  "\tThe default is '%3'.\n")
+            .arg(longRepresentation(),
+                 allDeprecationWarningModeStrings().join(QLatin1String("', '")),
+                 deprecationWarningModeName(defaultDeprecationWarningMode()));
+}
+
+QString DeprecationWarningsOption::longRepresentation() const
+{
+    return QStringLiteral("--deprecation-warnings");
+}
+
+void DeprecationWarningsOption::doParse(const QString &representation, QStringList &input)
+{
+    const QString mode = getArgument(representation, input);
+    if (mode.isEmpty()) {
+        throw ErrorInfo(Tr::tr("Invalid use of option '%1': No deprecation warning mode given.\n"
+                               "Usage: %2")
+                    .arg(representation, description(command())));
+    }
+
+    if (!allDeprecationWarningModeStrings().contains(mode)) {
+        throw ErrorInfo(Tr::tr("Invalid use of option '%1': "
+                               "Invalid deprecation warning mode '%2' given.\nUsage: %3")
+                        .arg(representation, mode, description(command())));
+    }
+
+    m_mode = deprecationWarningModeFromName(mode);
+}
+
 QString WaitLockOption::description(CommandType command) const
 {
     Q_UNUSED(command);
@@ -641,7 +707,30 @@ QString WaitLockOption::description(CommandType command) const
 
 QString WaitLockOption::longRepresentation() const
 {
-    return QLatin1String("--wait-lock");
+    return QStringLiteral("--wait-lock");
+}
+
+QString DisableFallbackProviderOption::description(CommandType) const
+{
+    return Tr::tr("%1\n\tDo not fall back to pkg-config if a dependency is not found.\n")
+            .arg(longRepresentation());
+}
+
+QString DisableFallbackProviderOption::longRepresentation() const
+{
+    return QStringLiteral("--no-fallback-module-provider");
+}
+
+QString RunEnvConfigOption::description(CommandType command) const
+{
+    Q_UNUSED(command);
+    return Tr::tr("%1\n\tComma-separated list of strings to pass to all modules' "
+                  "setupRunEnvironment scripts.\n").arg(longRepresentation());
+}
+
+QString RunEnvConfigOption::longRepresentation() const
+{
+    return QStringLiteral("--setup-run-env-config");
 }
 
 } // namespace qbs
