@@ -41,7 +41,7 @@
 
 #include "commandlineoption.h"
 
-#include <QHash>
+#include <QtCore/qhash.h>
 
 namespace qbs {
 
@@ -67,16 +67,19 @@ public:
     InstallRootOption *installRootOption() const;
     RemoveFirstOption *removeFirstoption() const;
     NoBuildOption *noBuildOption() const;
-    ForceOption *forceOption() const;
     ForceTimeStampCheckOption *forceTimestampCheckOption() const;
     ForceOutputCheckOption *forceOutputCheckOption() const;
     BuildNonDefaultOption *buildNonDefaultOption() const;
-    VersionOption *versionOption() const;
     LogTimeOption *logTimeOption() const;
     CommandEchoModeOption *commandEchoModeOption() const;
     SettingsDirOption *settingsDirOption() const;
+    JobLimitsOption *jobLimitsOption() const;
+    RespectProjectJobLimitsOption *respectProjectJobLimitsOption() const;
     GeneratorOption *generatorOption() const;
     WaitLockOption *waitLockOption() const;
+    DisableFallbackProviderOption *disableFallbackProviderOption() const;
+    RunEnvConfigOption *runEnvConfigOption() const;
+    DeprecationWarningsOption *deprecationWarningsOption() const;
 
 private:
     mutable QHash<CommandLineOption::Type, CommandLineOption *> m_options;
