@@ -58,6 +58,7 @@
 #include <tools/qttools.h>
 #include <tools/settings.h>
 #include <tools/settingsrepresentation.h>
+#include <tools/stlutils.h>
 
 #include <QtCore/qcoreapplication.h>
 #include <QtCore/qdir.h>
@@ -111,14 +112,9 @@ public:
     bool logTime;
 };
 
-CommandLineParser::CommandLineParser() : d(nullptr)
-{
-}
+CommandLineParser::CommandLineParser() = default;
 
-CommandLineParser::~CommandLineParser()
-{
-    delete d;
-}
+CommandLineParser::~CommandLineParser() = default;
 
 void CommandLineParser::printHelp() const
 {
@@ -282,6 +278,11 @@ QString CommandLineParser::settingsDir() const
     return d->settingsDir();
 }
 
+DeprecationWarningMode CommandLineParser::deprecationWarningMode() const
+{
+    return d->optionPool.deprecationWarningsOption()->mode();
+}
+
 QString CommandLineParser::commandName() const
 {
     return d->command->representation();
@@ -309,8 +310,7 @@ QList<QVariantMap> CommandLineParser::buildConfigurations() const
 
 bool CommandLineParser::parseCommandLine(const QStringList &args)
 {
-    delete d;
-    d = new CommandLineParserPrivate;
+    d = std::make_unique<CommandLineParserPrivate>();
     d->commandLine = args;
     try {
         d->doParse();
@@ -386,6 +386,7 @@ QList<Command *> CommandLineParser::CommandLineParserPrivate::allCommands() cons
             commandPool.getCommand(DumpNodesTreeCommandType),
             commandPool.getCommand(ListProductsCommandType),
             commandPool.getCommand(VersionCommandType),
+            commandPool.getCommand(SessionCommandType),
             commandPool.getCommand(HelpCommandType)};
 }
 
@@ -412,7 +413,7 @@ QString CommandLineParser::CommandLineParserPrivate::generalHelp() const
     for (const Command * command : commands)
         commandMap.insert(command->representation(), command);
 
-    for (const Command * command : qAsConst(commandMap)) {
+    for (const Command * command : std::as_const(commandMap)) {
         help.append(QLatin1String("  ")).append(command->representation());
         const QString whitespace
                 = QString(rhsIndentation - 2 - command->representation().size(), QLatin1Char(' '));
@@ -423,7 +424,7 @@ QString CommandLineParser::CommandLineParserPrivate::generalHelp() const
     toolNames.sort();
     if (!toolNames.empty()) {
         help.append(QLatin1Char('\n')).append(Tr::tr("Auxiliary commands:\n"));
-        for (const QString &toolName : qAsConst(toolNames)) {
+        for (const QString &toolName : std::as_const(toolNames)) {
             help.append(QLatin1String("  ")).append(toolName);
             const QString whitespace = QString(rhsIndentation - 2 - toolName.size(),
                                                QLatin1Char(' '));
@@ -506,7 +507,7 @@ void CommandLineParser::CommandLineParserPrivate::setupBuildConfigurations()
 
     const QVariantMap globalProperties = propertiesPerConfiguration.takeFirst().second;
     QList<QVariantMap> buildConfigs;
-    for (const PropertyListItem &item : qAsConst(propertiesPerConfiguration)) {
+    for (const PropertyListItem &item : std::as_const(propertiesPerConfiguration)) {
         QVariantMap properties = item.second;
         for (QVariantMap::ConstIterator globalPropIt = globalProperties.constBegin();
                  globalPropIt != globalProperties.constEnd(); ++globalPropIt) {
@@ -587,18 +588,15 @@ QString CommandLineParser::CommandLineParserPrivate::propertyName(const QString
     // Make fully-qualified, ie "platform" -> "qbs.platform"
     if (aCommandLineName.contains(QLatin1Char('.')))
         return aCommandLineName;
-    else
-        return QLatin1String("qbs.") + aCommandLineName;
+    return QLatin1String("qbs.") + aCommandLineName;
 }
 
 bool CommandLineParser::CommandLineParserPrivate::checkForExistingBuildConfiguration(
         const QList<QVariantMap> &buildConfigs, const QString &configurationName)
 {
-    for (const QVariantMap &buildConfig : buildConfigs) {
-        if (configurationName == getBuildConfigurationName(buildConfig))
-            return true;
-    }
-    return false;
+    return Internal::any_of(buildConfigs, [&configurationName](const auto &buildConfig) {
+        return configurationName == getBuildConfigurationName(buildConfig);
+    });
 }
 
 bool CommandLineParser::CommandLineParserPrivate::withNonDefaultProducts() const
