@@ -1,30 +1,39 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
-** This file is part of the Qt Build Suite.
+** This file is part of Qbs.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms and
-** conditions see http://www.qt.io/terms-conditions. For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
+**
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -41,10 +50,12 @@
 #include <parser/qmljslexer_p.h>
 #include <parser/qmljsparser_p.h>
 #include <tools/hostosinfo.h>
-#include <tools/scripttools.h>
+#include <tools/jsliterals.h>
 #include <tools/qbsassert.h>
+#include <tools/qttools.h>
+#include <tools/stringconstants.h>
 
-#include <QFile>
+#include <QtCore/qfile.h>
 
 using namespace QbsQmlJS;
 using namespace AST;
@@ -55,12 +66,12 @@ namespace Internal {
 class ItemFinder : public Visitor
 {
 public:
-    ItemFinder(const CodeLocation &cl) : m_cl(cl), m_item(0) { }
+    ItemFinder(CodeLocation cl) : m_cl(std::move(cl)), m_item(nullptr) { }
 
     UiObjectDefinition *item() const { return m_item; }
 
 private:
-    bool visit(UiObjectDefinition *ast)
+    bool visit(UiObjectDefinition *ast) override
     {
         if (toCodeLocation(m_cl.filePath(), ast->firstSourceLocation()) == m_cl) {
             m_item = ast;
@@ -77,23 +88,23 @@ class FilesBindingFinder : public Visitor
 {
 public:
     FilesBindingFinder(const UiObjectDefinition *startItem)
-        : m_startItem(startItem), m_binding(0)
+        : m_startItem(startItem), m_binding(nullptr)
     {
     }
 
     UiScriptBinding *binding() const { return m_binding; }
 
 private:
-    bool visit(UiObjectDefinition *ast)
+    bool visit(UiObjectDefinition *ast) override
     {
         // We start with the direct parent of the binding, so do not descend into any
         // other item.
         return ast == m_startItem;
     }
 
-    bool visit(UiScriptBinding *ast)
+    bool visit(UiScriptBinding *ast) override
     {
-        if (ast->qualifiedId->name.toString() != QLatin1String("files"))
+        if (ast->qualifiedId->name.toString() != StringConstants::filesProperty())
             return true;
         m_binding = ast;
         return false;
@@ -104,10 +115,12 @@ private:
 };
 
 
-ProjectFileUpdater::ProjectFileUpdater(const QString &projectFile) : m_projectFile(projectFile)
+ProjectFileUpdater::ProjectFileUpdater(QString projectFile) : m_projectFile(std::move(projectFile))
 {
 }
 
+ProjectFileUpdater::~ProjectFileUpdater() = default;
+
 ProjectFileUpdater::LineEndingType ProjectFileUpdater::guessLineEndingType(const QByteArray &text)
 {
     char before = 0;
@@ -157,7 +170,7 @@ void ProjectFileUpdater::apply()
     QByteArray rawContent = file.readAll();
     const LineEndingType origLineEndingType = guessLineEndingType(rawContent);
     convertToUnixLineEndings(&rawContent, origLineEndingType);
-    QString content = QString::fromLocal8Bit(rawContent);
+    QString content = QString::fromUtf8(rawContent);
 
     file.close();
     Engine engine;
@@ -166,10 +179,10 @@ void ProjectFileUpdater::apply()
     Parser parser(&engine);
     if (!parser.parse()) {
         QList<DiagnosticMessage> parserMessages = parser.diagnosticMessages();
-        if (!parserMessages.isEmpty()) {
+        if (!parserMessages.empty()) {
             ErrorInfo errorInfo;
             errorInfo.append(Tr::tr("Failure parsing project file."));
-            foreach (const DiagnosticMessage &msg, parserMessages)
+            for (const DiagnosticMessage &msg : std::as_const(parserMessages))
                 errorInfo.append(msg.message, toCodeLocation(file.fileName(), msg.loc));
             throw errorInfo;
         }
@@ -182,20 +195,28 @@ void ProjectFileUpdater::apply()
                         .arg(m_projectFile, file.errorString()));
     }
     file.resize(0);
-    rawContent = content.toLocal8Bit();
+    rawContent = content.toUtf8();
     convertFromUnixLineEndings(&rawContent, origLineEndingType);
     file.write(rawContent);
 }
 
 
-ProjectFileGroupInserter::ProjectFileGroupInserter(const ProductData &product,
-                                                   const QString &groupName)
+ProjectFileGroupInserter::ProjectFileGroupInserter(ProductData product, QString groupName)
     : ProjectFileUpdater(product.location().filePath())
-    , m_product(product)
-    , m_groupName(groupName)
+    , m_product(std::move(product))
+    , m_groupName(std::move(groupName))
 {
 }
 
+static int extractLine(const QString &fileContent, int pos)
+{
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 2)
+    return fileContent.leftRef(pos).count(QLatin1Char('\n'));
+#else
+    return QStringView{fileContent}.left(pos).count(QLatin1Char('\n'));
+#endif
+}
+
 void ProjectFileGroupInserter::doApply(QString &fileContent, UiProgram *ast)
 {
     ItemFinder itemFinder(m_product.location());
@@ -209,7 +230,7 @@ void ProjectFileGroupInserter::doApply(QString &fileContent, UiProgram *ast)
     Rewriter rewriter(fileContent, &changeSet, QStringList());
     QString groupItemString;
     const int productItemIndentation
-            = itemFinder.item()->qualifiedTypeNameId->firstSourceLocation().startColumn - 1;
+            = int(itemFinder.item()->qualifiedTypeNameId->firstSourceLocation().startColumn - 1);
     const int groupItemIndentation = productItemIndentation + 4;
     const QString groupItemIndentationString = QString(groupItemIndentation, QLatin1Char(' '));
     groupItemString += groupItemIndentationString + QLatin1String("Group {\n");
@@ -222,12 +243,12 @@ void ProjectFileGroupInserter::doApply(QString &fileContent, UiProgram *ast)
 
     int lineOffset = 3 + 1; // Our text + a leading newline that is always added by the rewriter.
     const QList<ChangeSet::EditOp> &editOps = changeSet.operationList();
-    QBS_CHECK(editOps.count() == 1);
-    const ChangeSet::EditOp &insertOp = editOps.first();
+    QBS_CHECK(editOps.size() == 1);
+    const ChangeSet::EditOp &insertOp = editOps.front();
     setLineOffset(lineOffset);
 
-    int insertionLine = fileContent.left(insertOp.pos1).count(QLatin1Char('\n'));
-    for (int i = 0; i < insertOp.text.count() && insertOp.text.at(i) == QLatin1Char('\n'); ++i)
+    int insertionLine = extractLine(fileContent, insertOp.pos1);
+    for (int i = 0; i < insertOp.text.size() && insertOp.text.at(i) == QLatin1Char('\n'); ++i)
         ++insertionLine; // To account for newlines prepended by the rewriter.
     ++insertionLine; // To account for zero-based indexing.
     setItemPosition(CodeLocation(projectFile(), insertionLine,
@@ -235,18 +256,18 @@ void ProjectFileGroupInserter::doApply(QString &fileContent, UiProgram *ast)
     changeSet.apply(&fileContent);
 }
 
-static QString getNodeRepresentation(const QString &fileContent, const Node *node)
+static QString getNodeRepresentation(const QString &fileContent, const QbsQmlJS::AST::Node *node)
 {
     const quint32 start = node->firstSourceLocation().offset;
     const quint32 end = node->lastSourceLocation().end();
-    return fileContent.mid(start, end - start);
+    return fileContent.mid(start, int(end - start));
 }
 
 static const ChangeSet::EditOp &getEditOp(const ChangeSet &changeSet)
 {
     const QList<ChangeSet::EditOp> &editOps = changeSet.operationList();
-    QBS_CHECK(editOps.count() == 1);
-    return editOps.first();
+    QBS_CHECK(editOps.size() == 1);
+    return editOps.front();
 }
 
 static int getLineOffsetForChangedBinding(const ChangeSet &changeSet, const QString &oldRhs)
@@ -256,16 +277,16 @@ static int getLineOffsetForChangedBinding(const ChangeSet &changeSet, const QStr
 
 static int getBindingLine(const ChangeSet &changeSet, const QString &fileContent)
 {
-    return fileContent.left(getEditOp(changeSet).pos1 + 1).count(QLatin1Char('\n')) + 1;
+    return extractLine(fileContent, getEditOp(changeSet).pos1 + 1) + 1;
 }
 
 
-ProjectFileFilesAdder::ProjectFileFilesAdder(const ProductData &product, const GroupData &group,
-                                             const QStringList &files)
+ProjectFileFilesAdder::ProjectFileFilesAdder(ProductData product, GroupData group,
+                                             QStringList files)
     : ProjectFileUpdater(product.location().filePath())
-    , m_product(product)
-    , m_group(group)
-    , m_files(files)
+    , m_product(std::move(product))
+    , m_group(std::move(group))
+    , m_files(std::move(files))
 {
 }
 
@@ -279,7 +300,7 @@ static QString &addToFilesRepr(QString &filesRepr, const QString &fileRepr, int
 
 static QString &addToFilesRepr(QString &filesRepr, const QStringList &filePaths, int indentation)
 {
-    foreach (const QString &f, filePaths)
+    for (const QString &f : filePaths)
         addToFilesRepr(filesRepr, toJSLiteral(f), indentation);
     return filesRepr;
 }
@@ -292,7 +313,7 @@ static QString &completeFilesRepr(QString &filesRepr, int indentation)
 
 void ProjectFileFilesAdder::doApply(QString &fileContent, UiProgram *ast)
 {
-    if (m_files.isEmpty())
+    if (m_files.empty())
         return;
     QStringList sortedFiles = m_files;
     sortedFiles.sort();
@@ -306,7 +327,7 @@ void ProjectFileFilesAdder::doApply(QString &fileContent, UiProgram *ast)
     }
 
     const int itemIndentation
-            = itemFinder.item()->qualifiedTypeNameId->firstSourceLocation().startColumn - 1;
+            = int(itemFinder.item()->qualifiedTypeNameId->firstSourceLocation().startColumn - 1);
     const int bindingIndentation = itemIndentation + 4;
     const int arrayElemIndentation = bindingIndentation + 4;
 
@@ -320,14 +341,13 @@ void ProjectFileFilesAdder::doApply(QString &fileContent, UiProgram *ast)
     UiScriptBinding * const filesBinding = bindingFinder.binding();
     if (filesBinding) {
         QString filesRepresentation;
-        if (filesBinding->statement->kind != Node::Kind_ExpressionStatement)
+        if (filesBinding->statement->kind != QbsQmlJS::AST::Node::Kind_ExpressionStatement)
             throw ErrorInfo(Tr::tr("JavaScript construct in source file is too complex.")); // TODO: rename, add new and concat.
-        const ExpressionStatement * const exprStatement
-                = static_cast<ExpressionStatement *>(filesBinding->statement);
+        const auto exprStatement
+                = static_cast<const ExpressionStatement *>(filesBinding->statement);
         switch (exprStatement->expression->kind) {
-        case Node::Kind_ArrayLiteral: {
-            const ElementList *elem
-                    = static_cast<ArrayLiteral *>(exprStatement->expression)->elements;
+        case QbsQmlJS::AST::Node::Kind_ArrayLiteral: {
+            auto elem = static_cast<const ArrayLiteral *>(exprStatement->expression)->elements;
             QStringList oldFileReprs;
             while (elem) {
                 oldFileReprs << getNodeRepresentation(fileContent, elem->expression);
@@ -335,21 +355,21 @@ void ProjectFileFilesAdder::doApply(QString &fileContent, UiProgram *ast)
             }
 
             // Insert new files "sorted", but do not change the order of existing files.
-            const QString firstNewFileRepr = toJSLiteral(sortedFiles.first());
-            while (!oldFileReprs.isEmpty()) {
-                if (oldFileReprs.first() > firstNewFileRepr)
+            const QString firstNewFileRepr = toJSLiteral(sortedFiles.front());
+            while (!oldFileReprs.empty()) {
+                if (oldFileReprs.front() > firstNewFileRepr)
                     break;
                 addToFilesRepr(filesRepresentation, oldFileReprs.takeFirst(), arrayElemIndentation);
             }
             addToFilesRepr(filesRepresentation, sortedFiles, arrayElemIndentation);
-            while (!oldFileReprs.isEmpty())
+            while (!oldFileReprs.empty())
                 addToFilesRepr(filesRepresentation, oldFileReprs.takeFirst(), arrayElemIndentation);
             completeFilesRepr(filesRepresentation, bindingIndentation);
             break;
         }
-        case Node::Kind_StringLiteral: {
-            const QString existingElement
-                    = static_cast<StringLiteral *>(exprStatement->expression)->value.toString();
+        case QbsQmlJS::AST::Node::Kind_StringLiteral: {
+            const auto existingElement
+                    = static_cast<const StringLiteral *>(exprStatement->expression)->value.toString();
             sortedFiles << existingElement;
             sortedFiles.sort();
             addToFilesRepr(filesRepresentation, sortedFiles, arrayElemIndentation);
@@ -368,18 +388,18 @@ void ProjectFileFilesAdder::doApply(QString &fileContent, UiProgram *ast)
 
             // It cannot be the other way around, since the existing right-hand side could
             // have string type.
-            filesRepresentation += QString::fromLatin1(".concat(%1)").arg(rhsRepr);
+            filesRepresentation += QStringLiteral(".concat(%1)").arg(rhsRepr);
 
         }
         }
-        rewriter.changeBinding(itemFinder.item()->initializer, QLatin1String("files"),
+        rewriter.changeBinding(itemFinder.item()->initializer, StringConstants::filesProperty(),
                                filesRepresentation, Rewriter::ScriptBinding);
     } else { // Can happen for the product itself, for which the "files" binding is not mandatory.
         QString filesRepresentation;
         addToFilesRepr(filesRepresentation, sortedFiles, arrayElemIndentation);
         completeFilesRepr(filesRepresentation, bindingIndentation);
         const QString bindingString = QString(bindingIndentation, QLatin1Char(' '))
-                + QLatin1String("files");
+                + StringConstants::filesProperty();
         rewriter.addBinding(itemFinder.item()->initializer, bindingString, filesRepresentation,
                             Rewriter::ScriptBinding);
     }
@@ -393,18 +413,18 @@ void ProjectFileFilesAdder::doApply(QString &fileContent, UiProgram *ast)
     changeSet.apply(&fileContent);
 }
 
-ProjectFileFilesRemover::ProjectFileFilesRemover(const ProductData &product, const GroupData &group,
-                                                 const QStringList &files)
+ProjectFileFilesRemover::ProjectFileFilesRemover(ProductData product, GroupData group,
+                                                 QStringList files)
     : ProjectFileUpdater(product.location().filePath())
-    , m_product(product)
-    , m_group(group)
-    , m_files(files)
+    , m_product(std::move(product))
+    , m_group(std::move(group))
+    , m_files(std::move(files))
 {
 }
 
 void ProjectFileFilesRemover::doApply(QString &fileContent, UiProgram *ast)
 {
-    if (m_files.isEmpty())
+    if (m_files.empty())
         return;
 
     // Find the item containing the "files" binding.
@@ -423,7 +443,7 @@ void ProjectFileFilesRemover::doApply(QString &fileContent, UiProgram *ast)
                         m_product.location());
     }
 
-    if (bindingFinder.binding()->statement->kind != Node::Kind_ExpressionStatement)
+    if (bindingFinder.binding()->statement->kind != QbsQmlJS::AST::Node::Kind_ExpressionStatement)
         throw ErrorInfo(Tr::tr("JavaScript construct in source file is too complex."));
     const CodeLocation bindingLocation
             = toCodeLocation(projectFile(), bindingFinder.binding()->firstSourceLocation());
@@ -432,56 +452,57 @@ void ProjectFileFilesRemover::doApply(QString &fileContent, UiProgram *ast)
     Rewriter rewriter(fileContent, &changeSet, QStringList());
 
     const int itemIndentation
-            = itemFinder.item()->qualifiedTypeNameId->firstSourceLocation().startColumn - 1;
+            = int(itemFinder.item()->qualifiedTypeNameId->firstSourceLocation().startColumn - 1);
     const int bindingIndentation = itemIndentation + 4;
     const int arrayElemIndentation = bindingIndentation + 4;
 
-    const ExpressionStatement * const exprStatement
-            = static_cast<ExpressionStatement *>(bindingFinder.binding()->statement);
+    const auto exprStatement
+            = static_cast<const ExpressionStatement *>(bindingFinder.binding()->statement);
     switch (exprStatement->expression->kind) {
-    case Node::Kind_ArrayLiteral: {
+    case QbsQmlJS::AST::Node::Kind_ArrayLiteral: {
         QStringList filesToRemove = m_files;
         QStringList newFilesList;
-        const ElementList *elem = static_cast<ArrayLiteral *>(exprStatement->expression)->elements;
+        auto elem = static_cast<const ArrayLiteral *>(exprStatement->expression)->elements;
         while (elem) {
-            if (elem->expression->kind != Node::Kind_StringLiteral) {
+            if (elem->expression->kind != QbsQmlJS::AST::Node::Kind_StringLiteral) {
                 throw ErrorInfo(Tr::tr("JavaScript construct in source file is too complex."),
                                 bindingLocation);
             }
-            const QString existingFile
-                    = static_cast<StringLiteral *>(elem->expression)->value.toString();
+            const auto existingFile
+                    = static_cast<const StringLiteral *>(elem->expression)->value.toString();
             if (!filesToRemove.removeOne(existingFile))
                 newFilesList << existingFile;
             elem = elem->next;
         }
-        if (!filesToRemove.isEmpty()) {
+        if (!filesToRemove.empty()) {
             throw ErrorInfo(Tr::tr("The following files were not found in the 'files' list: %1")
                             .arg(filesToRemove.join(QLatin1String(", "))), bindingLocation);
         }
-        QString filesString = QLatin1String("[\n");
-        foreach (const QString &file, newFilesList) {
+        QString filesString;
+        filesString += QLatin1String("[\n");
+        for (const QString &file : std::as_const(newFilesList)) {
             filesString += QString(arrayElemIndentation, QLatin1Char(' '));
-            filesString += QString::fromLocal8Bit("\"%1\",\n").arg(file);
+            filesString += QStringLiteral("\"%1\",\n").arg(file);
         }
         filesString += QString(bindingIndentation, QLatin1Char(' '));
         filesString += QLatin1Char(']');
-        rewriter.changeBinding(itemFinder.item()->initializer, QLatin1String("files"),
+        rewriter.changeBinding(itemFinder.item()->initializer, StringConstants::filesProperty(),
                                filesString, Rewriter::ScriptBinding);
         break;
     }
-    case Node::Kind_StringLiteral: {
-        if (m_files.count() != 1) {
+    case QbsQmlJS::AST::Node::Kind_StringLiteral: {
+        if (m_files.size() != 1) {
             throw ErrorInfo(Tr::tr("Was requested to remove %1 files, but there is only "
-                                   "one in the list.").arg(m_files.count()), bindingLocation);
+                                   "one in the list.").arg(m_files.size()), bindingLocation);
         }
-        const QString existingFile
-                = static_cast<StringLiteral *>(exprStatement->expression)->value.toString();
-        if (existingFile != m_files.first()) {
+        const auto existingFile
+                = static_cast<const StringLiteral *>(exprStatement->expression)->value.toString();
+        if (existingFile != m_files.front()) {
             throw ErrorInfo(Tr::tr("File '%1' could not be found in the 'files' list.")
-                            .arg(m_files.first()), bindingLocation);
+                            .arg(m_files.front()), bindingLocation);
         }
-        rewriter.changeBinding(itemFinder.item()->initializer, QLatin1String("files"),
-                               QLatin1String("[]"), Rewriter::ScriptBinding);
+        rewriter.changeBinding(itemFinder.item()->initializer, StringConstants::filesProperty(),
+                               StringConstants::emptyArrayValue(), Rewriter::ScriptBinding);
         break;
     }
     default:
@@ -499,10 +520,10 @@ void ProjectFileFilesRemover::doApply(QString &fileContent, UiProgram *ast)
 }
 
 
-ProjectFileGroupRemover::ProjectFileGroupRemover(const ProductData &product, const GroupData &group)
+ProjectFileGroupRemover::ProjectFileGroupRemover(ProductData product, GroupData group)
     : ProjectFileUpdater(product.location().filePath())
-    , m_product(product)
-    , m_group(group)
+    , m_product(std::move(product))
+    , m_group(std::move(group))
 {
 }
 
@@ -528,8 +549,8 @@ void ProjectFileGroupRemover::doApply(QString &fileContent, UiProgram *ast)
 
     setItemPosition(m_group.location());
     const QList<ChangeSet::EditOp> &editOps = changeSet.operationList();
-    QBS_CHECK(editOps.count() == 1);
-    const ChangeSet::EditOp &op = editOps.first();
+    QBS_CHECK(editOps.size() == 1);
+    const ChangeSet::EditOp &op = editOps.front();
     const QString removedText = fileContent.mid(op.pos1, op.length1);
     setLineOffset(-removedText.count(QLatin1Char('\n')));
 
