@@ -67,6 +67,7 @@
 #include <tools/qbsassert.h>
 #include <tools/qttools.h>
 #include <tools/settings.h>
+#include <tools/stlutils.h>
 #include <tools/stringconstants.h>
 
 #include <QtCore/qdir.h>
@@ -86,10 +87,10 @@ bool Executor::ComparePriority::operator() (const BuildGraphNode *x, const Build
 }
 
 
-Executor::Executor(const Logger &logger, QObject *parent)
+Executor::Executor(Logger logger, QObject *parent)
     : QObject(parent)
     , m_productInstaller(nullptr)
-    , m_logger(logger)
+    , m_logger(std::move(logger))
     , m_progressObserver(nullptr)
     , m_state(ExecutorIdle)
     , m_cancelationTimer(new QTimer(this))
@@ -102,11 +103,8 @@ Executor::Executor(const Logger &logger, QObject *parent)
 
 Executor::~Executor()
 {
-    // jobs must be destroyed before deleting the shared scan result cache
-    for (ExecutorJob *job : qAsConst(m_availableJobs))
-        delete job;
-    for (ExecutorJob *job : m_processingJobs.keys())
-        delete job;
+    // jobs must be destroyed before deleting the m_inputArtifactScanContext
+    m_allJobs.clear();
     delete m_inputArtifactScanContext;
     delete m_productInstaller;
 }
@@ -137,11 +135,9 @@ void Executor::retrieveSourceFileTimestamp(Artifact *artifact) const
 {
     QBS_CHECK(artifact->artifactType == Artifact::SourceFile);
 
-    if (m_buildOptions.changedFiles().empty())
-        artifact->setTimestamp(recursiveFileTime(artifact->filePath()));
-    else if (m_buildOptions.changedFiles().contains(artifact->filePath()))
+    if (m_buildOptions.changedFiles().contains(artifact->filePath()))
         artifact->setTimestamp(FileTime::currentTime());
-    else if (!artifact->timestamp().isValid())
+    else if (m_buildOptions.changedFiles().empty() || !artifact->timestamp().isValid())
         artifact->setTimestamp(recursiveFileTime(artifact->filePath()));
 
     artifact->timestampRetrieved = true;
@@ -152,7 +148,7 @@ void Executor::retrieveSourceFileTimestamp(Artifact *artifact) const
 void Executor::build()
 {
     try {
-        m_partialBuild = m_productsToBuild.size() != m_allProducts.size();
+        m_partialBuild = size_t(m_productsToBuild.size()) != m_allProducts.size();
         doBuild();
     } catch (const ErrorInfo &e) {
         handleError(e);
@@ -169,7 +165,7 @@ void Executor::setProject(const TopLevelProjectPtr &project)
         m_projectsByName.insert(std::make_pair(p->name, p.get()));
 }
 
-void Executor::setProducts(const std::vector<ResolvedProductPtr> &productsToBuild)
+void Executor::setProducts(const QVector<ResolvedProductPtr> &productsToBuild)
 {
     m_productsToBuild = productsToBuild;
     m_productsByName.clear();
@@ -180,7 +176,7 @@ void Executor::setProducts(const std::vector<ResolvedProductPtr> &productsToBuil
 class ProductPrioritySetter
 {
     const std::vector<ResolvedProductPtr> &m_allProducts;
-    unsigned int m_priority;
+    unsigned int m_priority = 0;
     Set<ResolvedProductPtr> m_seenProducts;
 public:
     ProductPrioritySetter(const std::vector<ResolvedProductPtr> &allProducts) // TODO: Use only products to build?
@@ -196,7 +192,7 @@ public:
                 allDependencies += dep;
         }
         const Set<ResolvedProductPtr> rootProducts
-                = Set<ResolvedProductPtr>::fromStdVector(m_allProducts) - allDependencies;
+                = rangeTo<Set<ResolvedProductPtr>>(m_allProducts) - allDependencies;
         m_priority = UINT_MAX;
         m_seenProducts.clear();
         for (const ResolvedProductPtr &rootProduct : rootProducts)
@@ -208,7 +204,7 @@ private:
     {
         if (!m_seenProducts.insert(product).second)
             return;
-        for (const ResolvedProductPtr &dependency : qAsConst(product->dependencies))
+        for (const ResolvedProductPtr &dependency : std::as_const(product->dependencies))
             traverse(dependency);
         if (!product->buildData)
             return;
@@ -242,8 +238,7 @@ void Executor::doBuild()
     const QStringList &filesToConsider = m_buildOptions.filesToConsider();
     if (!filesToConsider.empty()) {
         for (const QString &fileToConsider : filesToConsider) {
-            const QList<FileResourceBase *> &files
-                    = m_project->buildData->lookupFiles(fileToConsider);
+            const auto &files = m_project->buildData->lookupFiles(fileToConsider);
             for (const FileResourceBase * const file : files) {
                 if (file->fileType() != FileResourceBase::FileTypeArtifact)
                     continue;
@@ -266,8 +261,7 @@ void Executor::doBuild()
 
     doSanityChecks();
     QBS_CHECK(!m_project->buildData->evaluationContext);
-    m_project->buildData->evaluationContext
-            = RulesEvaluationContextPtr(new RulesEvaluationContext(m_logger));
+    m_project->buildData->evaluationContext = std::make_shared<RulesEvaluationContext>(m_logger);
     m_evalContext = m_project->buildData->evaluationContext;
 
     m_elapsedTimeRules = m_elapsedTimeScanners = m_elapsedTimeInstalling = 0;
@@ -334,7 +328,7 @@ void Executor::updateLeaves(BuildGraphNode *node, NodeSet &seenNodes)
     }
 
     bool isLeaf = true;
-    for (BuildGraphNode *child : qAsConst(node->children)) {
+    for (BuildGraphNode *child : std::as_const(node->children)) {
         if (child->buildState != BuildGraphNode::Built) {
             isLeaf = false;
             updateLeaves(child, seenNodes);
@@ -392,7 +386,7 @@ bool Executor::schedulingBlockedByJobLimit(const BuildGraphNode *node)
 {
     if (node->type() != BuildGraphNode::ArtifactNodeType)
         return false;
-    const Artifact * const artifact = static_cast<const Artifact *>(node);
+    const auto artifact = static_cast<const Artifact *>(node);
     if (artifact->artifactType == Artifact::SourceFile)
         return false;
 
@@ -402,8 +396,10 @@ bool Executor::schedulingBlockedByJobLimit(const BuildGraphNode *node)
         if (currentJobCount == 0)
             continue;
         const auto jobLimitIsExceeded = [currentJobCount, jobPool, this](const Transformer *t) {
-            const int maxJobCount = m_jobLimitsPerProduct.at(t->product().get())
-                    .getLimit(jobPool);
+            const auto it = m_jobLimitsPerProduct.find(t->product().get());
+            if (it == m_jobLimitsPerProduct.cend())
+                return false; // See checkNodeProduct() for why this is possible
+            const int maxJobCount = it->second.getLimit(jobPool);
             return maxJobCount > 0 && currentJobCount >= maxJobCount;
         };
 
@@ -412,7 +408,8 @@ bool Executor::schedulingBlockedByJobLimit(const BuildGraphNode *node)
         // running transformers.
         if (jobLimitIsExceeded(transformer))
             return true;
-        for (const ExecutorJob * const runningJob : m_processingJobs.keys()) {
+        const auto runningJobs = m_processingJobs.keys();
+        for (const ExecutorJob * const runningJob : runningJobs) {
             if (!runningJob->jobPools().contains(jobPool))
                 continue;
             const Transformer * const runningTransformer = runningJob->transformer();
@@ -454,7 +451,7 @@ bool Executor::isUpToDate(Artifact *artifact) const
             return false;
     }
 
-    for (FileDependency *fileDependency : qAsConst(artifact->fileDependencies)) {
+    for (FileDependency *fileDependency : std::as_const(artifact->fileDependencies)) {
         if (!fileDependency->timestamp().isValid()) {
             qCDebug(lcUpToDateCheck) << "file dependency doesn't exist"
                                      << fileDependency->filePath();
@@ -481,7 +478,7 @@ bool Executor::mustExecuteTransformer(const TransformerPtr &transformer) const
 
     bool hasAlwaysUpdatedArtifacts = false;
     bool hasUpToDateNotAlwaysUpdatedArtifacts = false;
-    for (Artifact *artifact : qAsConst(transformer->outputs)) {
+    for (Artifact *artifact : std::as_const(transformer->outputs)) {
         if (isUpToDate(artifact)) {
             if (artifact->alwaysUpdated)
                 hasAlwaysUpdatedArtifacts = true;
@@ -570,7 +567,7 @@ void Executor::finishJob(ExecutorJob *job, bool success)
     updateJobCounts(transformer.get(), -1);
     if (success) {
         m_project->buildData->setDirty();
-        for (Artifact * const artifact : qAsConst(transformer->outputs)) {
+        for (Artifact * const artifact : std::as_const(transformer->outputs)) {
             if (artifact->alwaysUpdated) {
                 artifact->setTimestamp(FileTime::currentTime());
                 for (Artifact * const parent : artifact->parentArtifacts())
@@ -623,14 +620,13 @@ void Executor::finishJob(ExecutorJob *job, bool success)
 
 static bool allChildrenBuilt(BuildGraphNode *node)
 {
-    return std::all_of(node->children.cbegin(), node->children.cend(),
-                       std::mem_fn(&BuildGraphNode::isBuilt));
+    return Internal::all_of(node->children, std::mem_fn(&BuildGraphNode::isBuilt));
 }
 
 void Executor::finishNode(BuildGraphNode *leaf)
 {
     leaf->buildState = BuildGraphNode::Built;
-    for (BuildGraphNode * const parent : qAsConst(leaf->parents)) {
+    for (BuildGraphNode * const parent : std::as_const(leaf->parents)) {
         if (parent->buildState != BuildGraphNode::Buildable) {
             qCDebug(lcExec).noquote() << "parent" << parent->toString()
                                       << "build state:" << toString(parent->buildState);
@@ -665,8 +661,8 @@ bool Executor::transformerHasMatchingOutputTags(const TransformerConstPtr &trans
     if (m_activeFileTags.empty())
         return true; // No filtering requested.
 
-    return std::any_of(transformer->outputs.cbegin(), transformer->outputs.cend(),
-                       [this](const Artifact *a) { return artifactHasMatchingOutputTags(a); });
+    return Internal::any_of(transformer->outputs, [this](const Artifact *a) {
+        return artifactHasMatchingOutputTags(a); });
 }
 
 bool Executor::artifactHasMatchingOutputTags(const Artifact *artifact) const
@@ -683,8 +679,9 @@ bool Executor::transformerHasMatchingInputFiles(const TransformerConstPtr &trans
         return false;
     if (transformer->inputs.empty())
         return true;
-    for (const Artifact * const input : qAsConst(transformer->inputs)) {
-        for (const QString &filePath : m_buildOptions.filesToConsider()) {
+    for (const Artifact * const input : std::as_const(transformer->inputs)) {
+        const auto files = m_buildOptions.filesToConsider();
+        for (const QString &filePath : files) {
             if (input->filePath() == filePath
                     || input->fileTags().intersects(m_tagsNeededForFilesToConsider)) {
                 return true;
@@ -698,7 +695,7 @@ bool Executor::transformerHasMatchingInputFiles(const TransformerConstPtr &trans
 void Executor::setupJobLimits()
 {
     Settings settings(m_buildOptions.settingsDirectory());
-    for (const ResolvedProductConstPtr &p : m_productsToBuild) {
+    for (const auto &p : std::as_const(m_productsToBuild)) {
         const Preferences prefs(&settings, p->profile());
         const JobLimits &jobLimitsFromSettings = prefs.jobLimits();
         JobLimits effectiveJobLimits;
@@ -725,7 +722,8 @@ void Executor::cancelJobs()
         return;
     qCDebug(lcExec) << "Canceling all jobs.";
     setState(ExecutorCanceling);
-    for (ExecutorJob *job : m_processingJobs.keys())
+    const auto jobs = m_processingJobs.keys();
+    for (ExecutorJob *job : jobs)
         job->cancel();
 }
 
@@ -734,7 +732,7 @@ void Executor::setupProgressObserver()
     if (!m_progressObserver)
         return;
     int totalEffort = 1; // For the effort after the last rule application;
-    for (const ResolvedProductConstPtr &product : m_productsToBuild) {
+    for (const auto &product : std::as_const(m_productsToBuild)) {
         QBS_CHECK(product->buildData);
         const auto filtered = filterByType<RuleNode>(product->buildData->allNodes());
         totalEffort += std::distance(filtered.begin(), filtered.end());
@@ -746,7 +744,7 @@ void Executor::doSanityChecks()
 {
     QBS_CHECK(m_project);
     QBS_CHECK(!m_productsToBuild.empty());
-    for (const ResolvedProductConstPtr &product : m_productsToBuild) {
+    for (const auto &product : std::as_const(m_productsToBuild)) {
         QBS_CHECK(product->buildData);
         QBS_CHECK(product->topLevelProject() == m_project.get());
     }
@@ -754,7 +752,8 @@ void Executor::doSanityChecks()
 
 void Executor::handleError(const ErrorInfo &error)
 {
-    for (const ErrorItem &ei : error.items())
+    const auto items = error.items();
+    for (const ErrorItem &ei : items)
         m_error.append(ei);
     if (m_processingJobs.empty())
         finish();
@@ -764,10 +763,13 @@ void Executor::handleError(const ErrorInfo &error)
 
 void Executor::addExecutorJobs()
 {
-    qCDebug(lcExec) << "preparing executor for" << m_buildOptions.maxJobCount()
-                    << "jobs in parallel";
-    for (int i = 1; i <= m_buildOptions.maxJobCount(); i++) {
-        const auto job = new ExecutorJob(m_logger, this);
+    const int count = m_buildOptions.maxJobCount();
+    qCDebug(lcExec) << "preparing executor for" << count << "jobs in parallel";
+    m_allJobs.reserve(count);
+    m_availableJobs.reserve(count);
+    for (int i = 1; i <= count; i++) {
+        m_allJobs.push_back(std::make_unique<ExecutorJob>(m_logger));
+        const auto job = m_allJobs.back().get();
         job->setMainThreadScriptEngine(m_evalContext->engine());
         job->setObjectName(QStringLiteral("J%1").arg(i));
         job->setDryRun(m_buildOptions.dryRun());
@@ -833,7 +835,7 @@ void Executor::rescueOldBuildData(Artifact *artifact, bool *childrenAdded = null
                 childrenToConnect.push_back(child);
         }
         for (const QString &depPath : rad.fileDependencies) {
-            const QList<FileResourceBase *> depList = m_project->buildData->lookupFiles(depPath);
+            const auto &depList = m_project->buildData->lookupFiles(depPath);
             if (depList.empty()) {
                 canRescue = false;
                 qCDebug(lcBuildGraph) << "File dependency" << depPath
@@ -913,7 +915,7 @@ bool Executor::checkForUnbuiltDependencies(Artifact *artifact)
 {
     bool buildingDependenciesFound = false;
     NodeSet unbuiltDependencies;
-    for (BuildGraphNode * const dependency : qAsConst(artifact->children)) {
+    for (BuildGraphNode * const dependency : std::as_const(artifact->children)) {
         switch (dependency->buildState) {
         case BuildGraphNode::Untouched:
         case BuildGraphNode::Buildable:
@@ -944,7 +946,7 @@ bool Executor::checkForUnbuiltDependencies(Artifact *artifact)
 
 void Executor::potentiallyRunTransformer(const TransformerPtr &transformer)
 {
-    for (Artifact * const output : qAsConst(transformer->outputs)) {
+    for (Artifact * const output : std::as_const(transformer->outputs)) {
         // Rescuing build data can introduce new dependencies, potentially delaying execution of
         // this transformer.
         bool childrenAddedDueToRescue;
@@ -967,7 +969,7 @@ void Executor::potentiallyRunTransformer(const TransformerPtr &transformer)
 
     const bool mustExecute = mustExecuteTransformer(transformer);
     if (mustExecute || m_buildOptions.forceTimestampCheck()) {
-        for (Artifact * const output : qAsConst(transformer->outputs)) {
+        for (Artifact * const output : std::as_const(transformer->outputs)) {
             // Scan all input artifacts. If new dependencies were found during scanning, delay
             // execution of this transformer.
             InputArtifactScanner scanner(output, m_inputArtifactScanContext, m_logger);
@@ -998,7 +1000,7 @@ void Executor::runTransformer(const TransformerPtr &transformer)
 
     // create the output directories
     if (!m_buildOptions.dryRun()) {
-        for (Artifact * const output : qAsConst(transformer->outputs)) {
+        for (Artifact * const output : std::as_const(transformer->outputs)) {
             QDir outDir = QFileInfo(output->filePath()).absoluteDir();
             if (!outDir.exists() && !outDir.mkpath(StringConstants::dot())) {
                     throw ErrorInfo(tr("Failed to create directory '%1'.")
@@ -1009,7 +1011,7 @@ void Executor::runTransformer(const TransformerPtr &transformer)
 
     QBS_CHECK(!m_availableJobs.empty());
     ExecutorJob *job = m_availableJobs.takeFirst();
-    for (Artifact * const artifact : qAsConst(transformer->outputs))
+    for (Artifact * const artifact : std::as_const(transformer->outputs))
         artifact->buildState = BuildGraphNode::Building;
     m_processingJobs.insert(job, transformer);
     updateJobCounts(transformer.get(), 1);
@@ -1019,7 +1021,7 @@ void Executor::runTransformer(const TransformerPtr &transformer)
 void Executor::finishTransformer(const TransformerPtr &transformer)
 {
     transformer->markedForRerun = false;
-    for (Artifact * const artifact : qAsConst(transformer->outputs)) {
+    for (Artifact * const artifact : std::as_const(transformer->outputs)) {
         possiblyInstallArtifact(artifact);
         finishArtifact(artifact);
     }
@@ -1054,7 +1056,7 @@ void Executor::onJobFinished(const qbs::ErrorInfo &err)
         if (m_evalContext->engine()->isActive()) {
             qCDebug(lcExec) << "Executor job finished while rule execution is pausing. "
                                "Delaying slot execution.";
-            QTimer::singleShot(0, job, [job, err] { job->finished(err); });
+            QTimer::singleShot(0, job, [job, err] { emit job->finished(err); });
             return;
         }
 
@@ -1079,10 +1081,10 @@ void Executor::checkForUnbuiltProducts()
 {
     if (m_buildOptions.executeRulesOnly())
         return;
-    QList<ResolvedProductPtr> unbuiltProducts;
-    for (const ResolvedProductPtr &product : m_productsToBuild) {
+    std::vector<ResolvedProductPtr> unbuiltProducts;
+    for (const ResolvedProductPtr &product : std::as_const(m_productsToBuild)) {
         bool productBuilt = true;
-        for (BuildGraphNode *rootNode : qAsConst(product->buildData->rootNodes())) {
+        for (BuildGraphNode *rootNode : std::as_const(product->buildData->rootNodes())) {
             if (rootNode->buildState != BuildGraphNode::Built) {
                 productBuilt = false;
                 unbuiltProducts.push_back(product);
@@ -1105,10 +1107,8 @@ void Executor::checkForUnbuiltProducts()
         m_logger.qbsInfo() << Tr::tr("Build done%1.").arg(configString());
     } else {
         m_error.append(Tr::tr("The following products could not be built%1:").arg(configString()));
-        QStringList productNames;
-        std::transform(unbuiltProducts.cbegin(), unbuiltProducts.cend(),
-                       std::back_inserter(productNames),
-                       [](const ResolvedProductConstPtr &p) { return p->fullDisplayName(); });
+        auto productNames = transformed<QStringList>(unbuiltProducts, [](const auto &p) {
+            return p->fullDisplayName(); });
         std::sort(productNames.begin(), productNames.end());
         m_error.append(productNames.join(QLatin1String(", ")));
     }
@@ -1198,11 +1198,11 @@ void Executor::prepareAllNodes()
     for (const ResolvedProductPtr &product : m_allProducts) {
         if (product->enabled) {
             QBS_CHECK(product->buildData);
-            for (BuildGraphNode * const node : qAsConst(product->buildData->allNodes()))
+            for (BuildGraphNode * const node : std::as_const(product->buildData->allNodes()))
                 node->buildState = BuildGraphNode::Untouched;
         }
     }
-    for (const ResolvedProductPtr &product : m_productsToBuild) {
+    for (const ResolvedProductPtr &product : std::as_const(m_productsToBuild)) {
         QBS_CHECK(product->buildData);
         for (Artifact * const artifact : filterByType<Artifact>(product->buildData->allNodes()))
             prepareArtifact(artifact);
@@ -1224,12 +1224,12 @@ void Executor::syncFileDependencies()
                                    "removing from lookup table";
         m_project->buildData->removeFromLookupTable(dep);
         bool isReferencedByArtifact = false;
-        for (const ResolvedProductConstPtr &product : m_allProducts) {
+        for (const auto &product : m_allProducts) {
             if (!product->buildData)
                 continue;
             const auto artifactList = filterByType<Artifact>(product->buildData->allNodes());
-            isReferencedByArtifact = std::any_of(artifactList.begin(), artifactList.end(),
-                    [dep](const Artifact *a) { return a->fileDependencies.contains(dep); });
+            isReferencedByArtifact = Internal::any_of(artifactList, [dep](const Artifact *a) {
+                return a->fileDependencies.contains(dep); });
             // TODO: Would it be safe to mark the artifact as "not up to date" here and clear
             //       its list of file dependencies, rather than doing the check again in
             //       isUpToDate()?
@@ -1286,7 +1286,7 @@ void Executor::setupForBuildingSelectedFiles(const BuildGraphNode *node)
  */
 void Executor::prepareReachableNodes()
 {
-    for (BuildGraphNode * const root : qAsConst(m_roots))
+    for (BuildGraphNode * const root : std::as_const(m_roots))
         prepareReachableNodes_impl(root);
 }
 
@@ -1298,7 +1298,7 @@ void Executor::prepareReachableNodes_impl(BuildGraphNode *node)
         return;
 
     node->buildState = BuildGraphNode::Buildable;
-    for (BuildGraphNode *child : qAsConst(node->children))
+    for (BuildGraphNode *child : std::as_const(node->children))
         prepareReachableNodes_impl(child);
 }
 
@@ -1306,7 +1306,7 @@ void Executor::prepareProducts()
 {
     ProductPrioritySetter prioritySetter(m_allProducts);
     prioritySetter.apply();
-    for (const ResolvedProductPtr &product : m_productsToBuild) {
+    for (const ResolvedProductPtr &product : std::as_const(m_productsToBuild)) {
         EnvironmentScriptRunner(product.get(), m_evalContext.get(), m_project->environment)
                 .setupForBuild();
     }
@@ -1315,7 +1315,7 @@ void Executor::prepareProducts()
 void Executor::setupRootNodes()
 {
     m_roots.clear();
-    for (const ResolvedProductPtr &product : m_productsToBuild)
+    for (const ResolvedProductPtr &product : std::as_const(m_productsToBuild))
         m_roots += product->buildData->rootNodes();
 }
 
