@@ -49,11 +49,12 @@
 
 #include <language/language.h>
 #include <logging/categories.h>
-#include <tools/fileinfo.h>
-#include <tools/scannerpluginmanager.h>
-#include <tools/qbsassert.h>
 #include <tools/error.h>
+#include <tools/fileinfo.h>
+#include <tools/qbsassert.h>
 #include <tools/qttools.h>
+#include <tools/scannerpluginmanager.h>
+#include <tools/stlutils.h>
 
 #include <QtCore/qdir.h>
 #include <QtCore/qstringlist.h>
@@ -157,7 +158,7 @@ void InputArtifactScanner::scan()
     for (Artifact * const dependency : childrenAddedByScanner)
         disconnect(m_artifact, dependency);
 
-    for (Artifact * const inputArtifact : qAsConst(m_artifact->transformer->inputs))
+    for (Artifact * const inputArtifact : std::as_const(m_artifact->transformer->inputs))
         scanForFileDependencies(inputArtifact);
 }
 
@@ -212,9 +213,9 @@ Set<DependencyScanner *> InputArtifactScanner::scannersForArtifact(const Artifac
         InputArtifactScannerContext::DependencyScannerCacheItem &cache = scannerCache[fileTag];
         if (!cache.valid) {
             cache.valid = true;
-            for (ScannerPlugin *scanner : ScannerPluginManager::scannersForFileTag(fileTag)) {
-                cache.scanners.push_back(std::make_shared<PluginDependencyScanner>(scanner));
-            }
+            const auto scanners = ScannerPluginManager::scannersForFileTag(fileTag);
+            transform(scanners, cache.scanners, [](const auto &scanner) {
+                return std::make_shared<PluginDependencyScanner>(scanner); });
             for (const ResolvedScannerConstPtr &scanner : product->scanners) {
                 if (scanner->inputs.contains(fileTag)) {
                     cache.scanners.push_back(
@@ -223,7 +224,7 @@ Set<DependencyScanner *> InputArtifactScanner::scannersForArtifact(const Artifac
                 }
             }
         }
-        for (const DependencyScannerPtr &scanner : qAsConst(cache.scanners))
+        for (const DependencyScannerPtr &scanner : std::as_const(cache.scanners))
             scanners += scanner.get();
     }
     return scanners;
@@ -242,7 +243,7 @@ void InputArtifactScanner::scanForScannerFileDependencies(DependencyScanner *sca
         cache.searchPaths = scanner->collectSearchPaths(inputArtifact);
     }
     qCDebug(lcDepScan) << "include paths (cache" << (cacheHit ? "hit)" : "miss)");
-    for (const QString &s : qAsConst(cache.searchPaths))
+    for (const QString &s : std::as_const(cache.searchPaths))
         qCDebug(lcDepScan) << "    " << s;
 
     const QString &filePathToBeScanned = fileToBeScanned->filePath();
@@ -288,7 +289,7 @@ void InputArtifactScanner::resolveScanResultDependencies(const Artifact *inputAr
         }
 
         // try include paths
-        for (const QString &includePath : qAsConst(cache.searchPaths)) {
+        for (const QString &includePath : std::as_const(cache.searchPaths)) {
             resolveDepencency(dependency, inputArtifact->product.get(),
                               &resolvedDependency, includePath);
             if (resolvedDependency.isValid())
