@@ -1,6 +1,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2016 The Qt Company Ltd.
+** Copyright (C) 2019 Jochen Ulrich <jochenulrich@t-online.de>
 ** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qbs.
@@ -60,19 +61,19 @@
 #include <tools/shellutils.h>
 #include <tools/stringconstants.h>
 
+#include <quickjs.h>
+
 #include <QtCore/qdir.h>
 #include <QtCore/qtemporaryfile.h>
 #include <QtCore/qtimer.h>
 
-#include <QtScript/qscriptvalue.h>
-
 namespace qbs {
 namespace Internal {
 
 ProcessCommandExecutor::ProcessCommandExecutor(const Logger &logger, QObject *parent)
     : AbstractCommandExecutor(logger, parent)
 {
-    connect(&m_process, static_cast<void (QbsProcess::*)(QProcess::ProcessError)>(&QbsProcess::error),
+    connect(&m_process, &QbsProcess::errorOccurred,
             this, &ProcessCommandExecutor::onProcessError);
     connect(&m_process, static_cast<void (QbsProcess::*)(int)>(&QbsProcess::finished),
             this, &ProcessCommandExecutor::onProcessFinished);
@@ -89,7 +90,8 @@ static QProcessEnvironment mergeEnvironments(const QProcessEnvironment &baseEnv,
         QStringLiteral("DYLD_LIBRARY_PATH"),
         QStringLiteral("DYLD_FRAMEWORK_PATH"),
     };
-    for (const QString &key : additionalEnv.keys()) {
+    const auto keys = additionalEnv.keys();
+    for (const QString &key : keys) {
         QString newValue = additionalEnv.value(key);
         if (pathListVariables.contains(key, HostOsInfo::fileNameCaseSensitivity())) {
             const QString &oldValue = baseEnv.value(key);
@@ -108,7 +110,8 @@ void ProcessCommandExecutor::doSetup()
                                              transformer()->product()->buildEnvironment)
             .findExecutable(cmd->program(), cmd->workingDir());
     cmd->clearRelevantEnvValues();
-    for (const QString &key : cmd->relevantEnvVars())
+    const auto keys = cmd->relevantEnvVars();
+    for (const QString &key : keys)
         cmd->addRelevantEnvValue(key, transformer()->product()->buildEnvironment.value(key));
 
     m_commandEnvironment = mergeEnvironments(m_buildEnvironment, cmd->environment());
@@ -117,9 +120,9 @@ void ProcessCommandExecutor::doSetup()
     m_shellInvocation = shellQuote(QDir::toNativeSeparators(m_program), m_arguments);
 }
 
-void ProcessCommandExecutor::doStart()
+bool ProcessCommandExecutor::doStart()
 {
-    QBS_ASSERT(m_process.state() == QProcess::NotRunning, return);
+    QBS_ASSERT(m_process.state() == QProcess::NotRunning, return false);
 
     const ProcessCommand * const cmd = processCommand();
 
@@ -129,7 +132,7 @@ void ProcessCommandExecutor::doStart()
 
     if (dryRun() && !cmd->ignoreDryRun()) {
         QTimer::singleShot(0, this, [this] { emit finished(); }); // Don't call back on the caller.
-        return;
+        return false;
     }
 
     const QString workingDir = QDir::fromNativeSeparators(cmd->workingDir());
@@ -140,7 +143,7 @@ void ProcessCommandExecutor::doStart()
                     "is invalid.").arg(QDir::toNativeSeparators(workingDir),
                                        QDir::toNativeSeparators(m_program)),
                                     cmd->codeLocation()));
-            return;
+            return false;
         }
     }
 
@@ -161,8 +164,9 @@ void ProcessCommandExecutor::doStart()
             if (!responseFile.open()) {
                 emit finished(ErrorInfo(Tr::tr("Cannot create response file '%1'.")
                                  .arg(responseFile.fileName())));
-                return;
+                return false;
             }
+            const auto separator = cmd->responseFileSeparator().toUtf8();
             for (int i = cmd->responseFileArgumentIndex(); i < cmd->arguments().size(); ++i) {
                 const QString arg = cmd->arguments().at(i);
                 if (arg.startsWith(cmd->responseFileUsagePrefix())) {
@@ -170,18 +174,18 @@ void ProcessCommandExecutor::doStart()
                     if (!f.open(QIODevice::ReadOnly)) {
                         emit finished(ErrorInfo(Tr::tr("Cannot open command file '%1'.")
                                                 .arg(QDir::toNativeSeparators(f.fileName()))));
-                        return;
+                        return false;
                     }
                     responseFile.write(f.readAll());
                 } else {
                     responseFile.write(qbs::Internal::shellQuote(arg).toLocal8Bit());
                 }
-                responseFile.write("\n");
+                responseFile.write(separator);
             }
             responseFile.close();
             m_responseFileName = responseFile.fileName();
-            arguments = arguments.mid(0,
-                                      std::min(cmd->responseFileArgumentIndex(), arguments.size()));
+            arguments = arguments.mid(0, std::min<int>(cmd->responseFileArgumentIndex(),
+                                                       arguments.size()));
             arguments += QDir::toNativeSeparators(cmd->responseFileUsagePrefix()
                     + responseFile.fileName());
         }
@@ -192,51 +196,58 @@ void ProcessCommandExecutor::doStart()
     qCDebug(lcExec) << "Additional environment:" << additionalVariables.toStringList();
     m_process.setWorkingDirectory(workingDir);
     m_process.start(m_program, arguments);
+    return true;
 }
 
-void ProcessCommandExecutor::cancel()
+void ProcessCommandExecutor::cancel(const qbs::ErrorInfo &reason)
 {
     // We don't want this command to be reported as failing, since we explicitly terminated it.
     disconnect(this, &ProcessCommandExecutor::reportProcessResult, nullptr, nullptr);
 
+    m_cancelReason = reason;
     m_process.cancel();
 }
 
 QString ProcessCommandExecutor::filterProcessOutput(const QByteArray &_output,
         const QString &filterFunctionSource)
 {
-    const QString output = QString::fromLocal8Bit(_output);
+    QString output = QString::fromLocal8Bit(_output);
     if (filterFunctionSource.isEmpty())
         return output;
 
-    QScriptValue scope = scriptEngine()->newObject();
-    scope.setPrototype(scriptEngine()->globalObject());
-    for (QVariantMap::const_iterator it = command()->properties().constBegin();
-            it != command()->properties().constEnd(); ++it) {
-        scope.setProperty(it.key(), scriptEngine()->toScriptValue(it.value()));
+    JSContext * const ctx = scriptEngine()->context();
+    const ScopedJsValue scope(ctx, JS_NewObjectProto(ctx, scriptEngine()->globalObject()));
+    for (auto it = command()->properties().constBegin();
+         it != command()->properties().constEnd(); ++it) {
+        setJsProperty(ctx, scope, it.key(), scriptEngine()->toScriptValue(it.value()));
     }
 
-    TemporaryGlobalObjectSetter tgos(scope);
-    QScriptValue filterFunction = scriptEngine()->evaluate(QLatin1String("var f = ")
-                                                           + filterFunctionSource
-                                                           + QLatin1String("; f"));
-    if (!filterFunction.isFunction()) {
+    TemporaryGlobalObjectSetter tgos(scriptEngine(), scope);
+    const ScopedJsValue filterFunction(
+                ctx,
+                scriptEngine()->evaluate(JsValueOwner::Caller,
+                                         QLatin1String("var f = ")
+                                         + filterFunctionSource
+                                         + QLatin1String("; f")));
+    if (!JS_IsFunction(scriptEngine()->context(), filterFunction)) {
         logger().printWarning(ErrorInfo(Tr::tr("Error in filter function: %1.\n%2")
-                         .arg(filterFunctionSource, filterFunction.toString())));
+                         .arg(filterFunctionSource,
+                              getJsString(scriptEngine()->context(), filterFunction))));
         return output;
     }
 
-    QScriptValue outputArg = scriptEngine()->newArray(1);
-    outputArg.setProperty(0, scriptEngine()->toScriptValue(output));
-    QScriptValue filteredOutput = filterFunction.call(scriptEngine()->undefinedValue(), outputArg);
-    if (scriptEngine()->hasErrorOrException(filteredOutput)) {
-        logger().printWarning(ErrorInfo(Tr::tr("Error when calling output filter function: %1")
-                         .arg(scriptEngine()->lastErrorString(filteredOutput)),
-                                        scriptEngine()->lastErrorLocation(filteredOutput)));
+    const ScopedJsValue outputArg(ctx, scriptEngine()->toScriptValue(output));
+    JSValue outputArgForCall = outputArg;
+    const ScopedJsValue filteredOutput(
+                ctx, JS_Call(ctx, filterFunction, JS_UNDEFINED, 1, &outputArgForCall));
+    if (const JsException ex = scriptEngine()->checkAndClearException({})) {
+        ErrorInfo err = ex.toErrorInfo();
+        err.prepend(Tr::tr("Error when calling output filter function"));
+        logger().printWarning(err);
         return output;
     }
 
-    return filteredOutput.toString();
+    return getJsString(ctx, filteredOutput);
 }
 
 static QProcess::ProcessError saveToFile(const QString &filePath, const QByteArray &content)
@@ -272,13 +283,15 @@ void ProcessCommandExecutor::getProcessOutput(bool stdOut, ProcessResult &result
     }
     QString contentString = filterProcessOutput(content, filterFunction);
     if (!redirectPath.isEmpty()) {
-        const QProcess::ProcessError error = saveToFile(redirectPath, contentString.toLocal8Bit());
+        const QByteArray dataToWrite = filterFunction.isEmpty() ? content
+                                                                : contentString.toLocal8Bit();
+        const QProcess::ProcessError error = saveToFile(redirectPath, dataToWrite);
         if (result.error() == QProcess::UnknownError && error != QProcess::UnknownError)
             result.d->error = error;
     } else {
         if (!contentString.isEmpty() && contentString.endsWith(QLatin1Char('\n')))
             contentString.chop(1);
-        *target = contentString.split(QLatin1Char('\n'), QString::SkipEmptyParts);
+        *target = contentString.split(QLatin1Char('\n'), QBS_SKIP_EMPTY_PARTS);
     }
 }
 
@@ -300,10 +313,13 @@ void ProcessCommandExecutor::sendProcessOutput()
     const bool processError = result.error() != QProcess::UnknownError;
     const bool failureExit = quint32(m_process.exitCode())
             > quint32(processCommand()->maxExitCode());
-    result.d->success = !processError && !failureExit;
+    const bool cancelledWithError = m_cancelReason.hasError();
+    result.d->success = !processError && !failureExit && !cancelledWithError;
     emit reportProcessResult(result);
 
-    if (Q_UNLIKELY(processError)) {
+    if (Q_UNLIKELY(cancelledWithError)) {
+        emit finished(m_cancelReason);
+    } else if (Q_UNLIKELY(processError)) {
         emit finished(ErrorInfo(errorString));
     } else if (Q_UNLIKELY(failureExit)) {
         emit finished(ErrorInfo(Tr::tr("Process failed with exit code %1.")
@@ -321,6 +337,8 @@ void ProcessCommandExecutor::onProcessError()
         QTimer::singleShot(0, this, &ProcessCommandExecutor::onProcessError);
         return;
     }
+    if (m_cancelReason.hasError())
+        return; // Ignore. Cancel reasons will be handled by on ProcessFinished().
     switch (m_process.error()) {
     case QProcess::FailedToStart: {
         removeResponseFile();
@@ -380,7 +398,7 @@ void ProcessCommandExecutor::doReportCommandDescription(const QString &productNa
         if (m_echoMode == CommandEchoModeCommandLineWithEnvironment) {
             QStringList keys = m_commandEnvironment.keys();
             keys.sort();
-            for (const QString &key : keys)
+            for (const QString &key : std::as_const(keys))
                 fullInvocation += environmentVariableString(key, m_commandEnvironment.value(key));
         }
         fullInvocation += m_shellInvocation;
