@@ -60,6 +60,8 @@
 #include <tools/qttools.h>
 #include <tools/stlutils.h>
 
+#include <memory>
+
 namespace qbs {
 namespace Internal {
 
@@ -96,8 +98,7 @@ QString ProjectBuildData::deriveBuildGraphFilePath(const QString &buildDir, cons
 
 void ProjectBuildData::insertIntoLookupTable(FileResourceBase *fileres)
 {
-    QList<FileResourceBase *> &lst
-            = m_artifactLookupTable[fileres->fileName()][fileres->dirPath()];
+    auto &lst = m_artifactLookupTable[{fileres->fileName(), fileres->dirPath()}];
     const auto * const artifact = fileres->fileType() == FileResourceBase::FileTypeArtifact
             ? static_cast<Artifact *>(fileres) : nullptr;
     if (artifact && artifact->artifactType == Artifact::Generated) {
@@ -117,30 +118,32 @@ void ProjectBuildData::insertIntoLookupTable(FileResourceBase *fileres)
             throw error;
         }
     }
-    QBS_CHECK(!lst.contains(fileres));
+    QBS_CHECK(!contains(lst, fileres));
     lst.push_back(fileres);
     m_isDirty = true;
 }
 
 void ProjectBuildData::removeFromLookupTable(FileResourceBase *fileres)
 {
-    m_artifactLookupTable[fileres->fileName()][fileres->dirPath()].removeOne(fileres);
+    removeOne(m_artifactLookupTable[{fileres->fileName(), fileres->dirPath()}], fileres);
 }
 
-QList<FileResourceBase *> ProjectBuildData::lookupFiles(const QString &filePath) const
+const std::vector<FileResourceBase *> &ProjectBuildData::lookupFiles(const QString &filePath) const
 {
     QString dirPath, fileName;
     FileInfo::splitIntoDirectoryAndFileName(filePath, &dirPath, &fileName);
     return lookupFiles(dirPath, fileName);
 }
 
-QList<FileResourceBase *> ProjectBuildData::lookupFiles(const QString &dirPath,
+const std::vector<FileResourceBase *> &ProjectBuildData::lookupFiles(const QString &dirPath,
         const QString &fileName) const
 {
-    return m_artifactLookupTable.value(fileName).value(dirPath);
+    static const std::vector<FileResourceBase *> emptyResult;
+    const auto it = m_artifactLookupTable.find({fileName, dirPath});
+    return it != m_artifactLookupTable.end() ? it->second : emptyResult;
 }
 
-QList<FileResourceBase *> ProjectBuildData::lookupFiles(const Artifact *artifact) const
+const std::vector<FileResourceBase *> &ProjectBuildData::lookupFiles(const Artifact *artifact) const
 {
     return lookupFiles(artifact->dirPath(), artifact->fileName());
 }
@@ -154,7 +157,7 @@ void ProjectBuildData::insertFileDependency(FileDependency *dependency)
 static void disconnectArtifactChildren(Artifact *artifact)
 {
     qCDebug(lcBuildGraph) << "disconnect children of" << relativeArtifactFileName(artifact);
-    for (BuildGraphNode * const child : qAsConst(artifact->children))
+    for (BuildGraphNode * const child : std::as_const(artifact->children))
         child->parents.remove(artifact);
     artifact->children.clear();
     artifact->childrenAddedByScanner.clear();
@@ -163,7 +166,7 @@ static void disconnectArtifactChildren(Artifact *artifact)
 static void disconnectArtifactParents(Artifact *artifact)
 {
     qCDebug(lcBuildGraph) << "disconnect parents of" << relativeArtifactFileName(artifact);
-    for (BuildGraphNode * const parent : qAsConst(artifact->parents)) {
+    for (BuildGraphNode * const parent : std::as_const(artifact->parents)) {
         parent->children.remove(artifact);
         if (parent->type() != BuildGraphNode::ArtifactNodeType)
             continue;
@@ -237,7 +240,6 @@ void ProjectBuildData::removeArtifact(Artifact *artifact,
         artifact->transformer->outputs.remove(artifact);
     if (removeFromProduct)
         artifact->product->buildData->removeArtifact(artifact);
-    m_isDirty = false;
 }
 
 void ProjectBuildData::setDirty()
@@ -255,7 +257,7 @@ void ProjectBuildData::setClean()
 void ProjectBuildData::load(PersistentPool &pool)
 {
     serializationOp<PersistentPool::Load>(pool);
-    for (FileDependency * const dep : qAsConst(fileDependencies))
+    for (FileDependency * const dep : std::as_const(fileDependencies))
         insertIntoLookupTable(dep);
     m_isDirty = false;
 }
@@ -266,7 +268,7 @@ void ProjectBuildData::store(PersistentPool &pool)
 }
 
 
-BuildDataResolver::BuildDataResolver(const Logger &logger) : m_logger(logger)
+BuildDataResolver::BuildDataResolver(Logger logger) : m_logger(std::move(logger))
 {
 }
 
@@ -275,7 +277,7 @@ void BuildDataResolver::resolveBuildData(const TopLevelProjectPtr &resolvedProje
 {
     QBS_CHECK(!resolvedProject->buildData);
     m_project = resolvedProject;
-    resolvedProject->buildData.reset(new ProjectBuildData);
+    resolvedProject->buildData = std::make_unique<ProjectBuildData>();
     resolvedProject->buildData->evaluationContext = evalContext;
     const std::vector<ResolvedProductPtr> &allProducts = resolvedProject->allProducts();
     evalContext->initializeObserver(Tr::tr("Setting up build graph for configuration %1")
@@ -335,7 +337,7 @@ private:
     {
         if (!m_rulesOnPath.insert(rule.get()).second) {
             QString pathstr;
-            for (const Rule *r : qAsConst(m_rulePath)) {
+            for (const Rule *r : std::as_const(m_rulePath)) {
                 pathstr += QLatin1Char('\n') + r->toString() + QLatin1Char('\t')
                         + r->prepareScript.location().toString();
             }
@@ -389,10 +391,10 @@ void BuildDataResolver::resolveProductBuildData(const ResolvedProductPtr &produc
 
     evalContext()->checkForCancelation();
 
-    product->buildData.reset(new ProductBuildData);
+    product->buildData = std::make_unique<ProductBuildData>();
     ArtifactSetByFileTag artifactsPerFileTag;
 
-    for (const auto &dependency : qAsConst(product->dependencies)) {
+    for (const auto &dependency : std::as_const(product->dependencies)) {
         QBS_CHECK(dependency->enabled);
         resolveProductBuildData(dependency);
     }
@@ -410,7 +412,7 @@ void BuildDataResolver::resolveProductBuildData(const ResolvedProductPtr &produc
     artifactsPerFileTag["qbs"].insert(qbsFileArtifact);
 
     // read sources
-    for (const SourceArtifactConstPtr &sourceArtifact : product->allEnabledFiles()) {
+    for (const auto &sourceArtifact : product->allEnabledFiles()) {
         QString filePath = sourceArtifact->absoluteFilePath;
         if (lookupArtifact(product, filePath))
             continue; // ignore duplicate artifacts
@@ -442,7 +444,7 @@ void BuildDataResolver::connectRulesToDependencies(const ResolvedProductPtr &pro
     std::vector<RuleNode *> ruleNodes;
     for (RuleNode *ruleNode : filterByType<RuleNode>(product->buildData->allNodes()))
         ruleNodes.push_back(ruleNode);
-    for (const ResolvedProductConstPtr &dep : dependencies) {
+    for (const auto &dep : dependencies) {
         if (!dep->buildData)
             continue;
         for (RuleNode *depRuleNode : filterByType<RuleNode>(dep->buildData->allNodes())) {
@@ -471,7 +473,7 @@ ScriptEngine *BuildDataResolver::engine() const
     return evalContext()->engine();
 }
 
-QScriptValue BuildDataResolver::scope() const
+JSValue BuildDataResolver::scope() const
 {
     return evalContext()->scope();
 }
