@@ -56,13 +56,17 @@
 #include <tools/qbs_export.h>
 #include <QtCore/qstring.h>
 
+#if (QT_VERSION >= QT_VERSION_CHECK(6, 0, 0))
+#include <QtCore5Compat/qstringref.h>
+#endif
+
 namespace QbsQmlJS {
 
 class Engine;
 
 class QML_PARSER_EXPORT Directives {
 public:
-    virtual ~Directives() {}
+    virtual ~Directives() = default;
 
     virtual void pragmaLibrary()
     {
