@@ -40,17 +40,22 @@
 
 #include "architectures.h"
 
+#include "stringconstants.h"
+
 #include <QtCore/qmap.h>
 #include <QtCore/qstringlist.h>
 
 namespace qbs {
 
+using namespace Internal;
+
 QString canonicalTargetArchitecture(const QString &architecture,
+                                    const QString &endianness,
                                     const QString &vendor,
                                     const QString &system,
                                     const QString &abi)
 {
-    const QString arch = canonicalArchitecture(architecture);
+    QString arch = canonicalArchitecture(architecture);
     const bool isApple = (vendor == QStringLiteral("apple")
                           || system == QStringLiteral("darwin")
                           || system == QStringLiteral("macosx")
@@ -63,62 +68,82 @@ QString canonicalTargetArchitecture(const QString &architecture,
 
     if (arch == QStringLiteral("armv7a")) {
         if (isApple)
-            return QStringLiteral("armv7");
+            return StringConstants::armv7Arch();
         if (isQnx)
-            return QStringLiteral("arm");
+            return StringConstants::armArch();
     }
 
-    if (arch == QStringLiteral("arm64") && isQnx)
-        return QStringLiteral("aarch64");
+    if (arch == StringConstants::arm64Arch() && isQnx)
+        return StringConstants::aarch64Arch();
 
-    if (arch == QStringLiteral("x86")) {
+    if (arch == StringConstants::x86Arch()) {
         if (isQnx)
-            return QStringLiteral("i586");
-        return QStringLiteral("i386");
+            return StringConstants::i586Arch();
+        return StringConstants::i386Arch();
     }
 
+    if (arch == StringConstants::mipsArch() || arch == StringConstants::mips64Arch()) {
+        if (endianness == QStringLiteral("big"))
+            return arch + QStringLiteral("eb");
+        if (endianness == QStringLiteral("little"))
+            return arch + QStringLiteral("el");
+    }
+
+    if (arch == StringConstants::ppcArch())
+        return StringConstants::powerPcArch();
+
+    if (arch == StringConstants::ppc64Arch() && endianness == QStringLiteral("little"))
+        return arch + QStringLiteral("le");
+
     return arch;
 }
 
 QString canonicalArchitecture(const QString &architecture)
 {
     QMap<QString, QStringList> archMap;
-    archMap.insert(QLatin1String("x86"), QStringList()
-        << QLatin1String("i386")
-        << QLatin1String("i486")
-        << QLatin1String("i586")
-        << QLatin1String("i686")
-        << QLatin1String("ia32")
-        << QLatin1String("ia-32")
-        << QLatin1String("x86_32")
-        << QLatin1String("x86-32")
-        << QLatin1String("intel32")
-        << QLatin1String("mingw32"));
+    archMap.insert(StringConstants::x86Arch(), QStringList()
+        << StringConstants::i386Arch()
+        << QStringLiteral("i486")
+        << StringConstants::i586Arch()
+        << QStringLiteral("i686")
+        << QStringLiteral("ia32")
+        << QStringLiteral("ia-32")
+        << QStringLiteral("x86_32")
+        << QStringLiteral("x86-32")
+        << QStringLiteral("intel32")
+        << QStringLiteral("mingw32"));
 
-    archMap.insert(QLatin1String("x86_64"), QStringList()
-        << QLatin1String("x86-64")
-        << QLatin1String("x64")
-        << QLatin1String("amd64")
-        << QLatin1String("ia32e")
-        << QLatin1String("em64t")
-        << QLatin1String("intel64")
-        << QLatin1String("mingw64"));
+    archMap.insert(StringConstants::x86_64Arch(), QStringList()
+        << QStringLiteral("x86-64")
+        << QStringLiteral("x64")
+        << StringConstants::amd64Arch()
+        << QStringLiteral("ia32e")
+        << QStringLiteral("em64t")
+        << QStringLiteral("intel64")
+        << QStringLiteral("mingw64"));
 
-    archMap.insert(QLatin1String("arm64"), QStringList()
-        << QLatin1String("aarch64"));
+    archMap.insert(StringConstants::arm64Arch(), QStringList()
+        << StringConstants::aarch64Arch());
 
-    archMap.insert(QLatin1String("ia64"), QStringList()
-        << QLatin1String("ia-64")
-        << QLatin1String("itanium"));
+    archMap.insert(QStringLiteral("ia64"), QStringList()
+        << QStringLiteral("ia-64")
+        << QStringLiteral("itanium"));
 
-    archMap.insert(QLatin1String("ppc"), QStringList()
-        << QLatin1String("powerpc"));
+    archMap.insert(StringConstants::ppcArch(), QStringList()
+        << StringConstants::powerPcArch());
 
-    archMap.insert(QLatin1String("ppc64"), QStringList()
-        << QLatin1String("powerpc64"));
+    archMap.insert(StringConstants::ppc64Arch(), QStringList()
+        << QStringLiteral("ppc64le")
+        << QStringLiteral("powerpc64")
+        << QStringLiteral("powerpc64le"));
 
-    archMap.insert(QLatin1String("ppc64le"), QStringList()
-        << QLatin1String("powerpc64le"));
+    archMap.insert(StringConstants::mipsArch(), QStringList()
+        << QStringLiteral("mipseb")
+        << QStringLiteral("mipsel"));
+
+    archMap.insert(StringConstants::mips64Arch(), QStringList()
+        << QStringLiteral("mips64eb")
+        << QStringLiteral("mips64el"));
 
     QMapIterator<QString, QStringList> i(archMap);
     while (i.hasNext()) {
