@@ -42,14 +42,17 @@
 #include <logging/translator.h>
 #include <tools/buildgraphlocker.h>
 #include <tools/installoptions.h>
+#include <tools/jsonhelper.h>
 #include <tools/profile.h>
 #include <tools/qbsassert.h>
 #include <tools/scripttools.h>
 #include <tools/settings.h>
+#include <tools/stringconstants.h>
 
 #include <QtCore/qdir.h>
 #include <QtCore/qfileinfo.h>
 #include <QtCore/qprocess.h>
+#include <QtCore/qjsonobject.h>
 
 namespace qbs {
 namespace Internal {
@@ -69,14 +72,14 @@ public:
         , forceProbeExecution(false)
         , waitLockBuildGraph(false)
         , restoreBehavior(SetupProjectParameters::RestoreAndTrackChanges)
-        , propertyCheckingMode(ErrorHandlingMode::Relaxed)
+        , propertyCheckingMode(ErrorHandlingMode::Strict)
         , productErrorMode(ErrorHandlingMode::Strict)
     {
     }
 
     QString projectFilePath;
     QString topLevelProfile;
-    QString configurationName;
+    QString configurationName = QLatin1String("default");
     QString buildRoot;
     QStringList searchPaths;
     QStringList pluginPaths;
@@ -96,28 +99,76 @@ public:
     SetupProjectParameters::RestoreBehavior restoreBehavior;
     ErrorHandlingMode propertyCheckingMode;
     ErrorHandlingMode productErrorMode;
+    DeprecationWarningMode deprecationWarningMode = defaultDeprecationWarningMode();
     QProcessEnvironment environment;
 };
 
 } // namespace Internal
 
-SetupProjectParameters::SetupProjectParameters() : d(new Internal::SetupProjectParametersPrivate)
+using namespace Internal;
+
+SetupProjectParameters::SetupProjectParameters() : d(new SetupProjectParametersPrivate)
 {
 }
 
-SetupProjectParameters::SetupProjectParameters(const SetupProjectParameters &other) : d(other.d)
+SetupProjectParameters::SetupProjectParameters(const SetupProjectParameters &other) = default;
+
+SetupProjectParameters::SetupProjectParameters(
+        SetupProjectParameters &&other) Q_DECL_NOEXCEPT = default;
+
+SetupProjectParameters::~SetupProjectParameters() = default;
+
+SetupProjectParameters &SetupProjectParameters::operator=(
+        const SetupProjectParameters &other) = default;
+
+namespace Internal {
+template<> ErrorHandlingMode fromJson(const QJsonValue &v)
 {
+    if (v.toString() == QLatin1String("relaxed"))
+        return ErrorHandlingMode::Relaxed;
+    return ErrorHandlingMode::Strict;
 }
 
-SetupProjectParameters::~SetupProjectParameters()
+template<> DeprecationWarningMode fromJson(const QJsonValue &v)
 {
+    return deprecationWarningModeFromName(v.toString());
 }
 
-SetupProjectParameters &SetupProjectParameters::operator=(const SetupProjectParameters &other)
+template<> SetupProjectParameters::RestoreBehavior fromJson(const QJsonValue &v)
 {
-    d = other.d;
-    return *this;
+    const QString value = v.toString();
+    if (value == QLatin1String("restore-only"))
+        return SetupProjectParameters::RestoreOnly;
+    if (value == QLatin1String("resolve-only"))
+        return SetupProjectParameters::ResolveOnly;
+    return SetupProjectParameters::RestoreAndTrackChanges;
 }
+} // namespace Internal
+
+SetupProjectParameters SetupProjectParameters::fromJson(const QJsonObject &data)
+{
+    using namespace Internal;
+    SetupProjectParameters params;
+    setValueFromJson(params.d->topLevelProfile, data, "top-level-profile");
+    setValueFromJson(params.d->configurationName, data, "configuration-name");
+    setValueFromJson(params.d->projectFilePath, data, "project-file-path");
+    setValueFromJson(params.d->buildRoot, data, "build-root");
+    setValueFromJson(params.d->settingsBaseDir, data, "settings-directory");
+    setValueFromJson(params.d->overriddenValues, data, "overridden-properties");
+    setValueFromJson(params.d->dryRun, data, "dry-run");
+    setValueFromJson(params.d->logElapsedTime, data, "log-time");
+    setValueFromJson(params.d->forceProbeExecution, data, "force-probe-execution");
+    setValueFromJson(params.d->waitLockBuildGraph, data, "wait-lock-build-graph");
+    setValueFromJson(params.d->fallbackProviderEnabled, data, "fallback-provider-enabled");
+    setValueFromJson(params.d->environment, data, "environment");
+    setValueFromJson(params.d->restoreBehavior, data, "restore-behavior");
+    setValueFromJson(params.d->propertyCheckingMode, data, "error-handling-mode");
+    setValueFromJson(params.d->deprecationWarningMode, data, "deprecation-warning-mode");
+    params.d->productErrorMode = params.d->propertyCheckingMode;
+    return params;
+}
+
+SetupProjectParameters &SetupProjectParameters::operator=(SetupProjectParameters &&other) Q_DECL_NOEXCEPT = default;
 
 /*!
  * \brief Returns the name of the top-level profile for building the project.
@@ -178,6 +229,44 @@ void SetupProjectParameters::setProjectFilePath(const QString &projectFilePath)
         d->projectFilePath = canonicalProjectFilePath;
 }
 
+void SetupProjectParameters::finalizeProjectFilePath()
+{
+    QString filePath = projectFilePath();
+    if (filePath.isEmpty())
+        filePath = QDir::currentPath();
+    const QFileInfo projectFileInfo(filePath);
+    if (!projectFileInfo.exists())
+        throw ErrorInfo(Tr::tr("Project file '%1' cannot be found.").arg(filePath));
+    if (projectFileInfo.isRelative())
+        filePath = projectFileInfo.absoluteFilePath();
+    if (projectFileInfo.isFile()) {
+        setProjectFilePath(filePath);
+        return;
+    }
+    if (!projectFileInfo.isDir())
+        throw ErrorInfo(Tr::tr("Project file '%1' has invalid type.").arg(filePath));
+
+    const QStringList &actualFileNames
+        = QDir(filePath).entryList(StringConstants::qbsFileWildcards(), QDir::Files);
+    if (actualFileNames.empty()) {
+        QString error;
+        if (projectFilePath().isEmpty())
+            error = Tr::tr("No project file given and none found in current directory.\n");
+        else
+            error = Tr::tr("No project file found in directory '%1'.").arg(filePath);
+        throw ErrorInfo(error);
+    }
+    if (actualFileNames.size() > 1) {
+        throw ErrorInfo(Tr::tr("More than one project file found in directory '%1'.")
+                            .arg(filePath));
+    }
+    filePath.append(QLatin1Char('/')).append(actualFileNames.front());
+
+    filePath = QDir::current().filePath(filePath);
+    filePath = QDir::cleanPath(filePath);
+    setProjectFilePath(filePath);
+}
+
 /*!
  * \brief Returns the base path of where to put the build artifacts and store the build graph.
  */
@@ -202,7 +291,7 @@ void SetupProjectParameters::setBuildRoot(const QString &buildRoot)
     // Calling mkpath() may be necessary to get the canonical build root, but if we do it,
     // it must be reverted immediately afterwards as not to create directories needlessly,
     // e.g in the case of a dry run build.
-    Internal::DirectoryManager dirManager(buildRoot, Internal::Logger());
+    DirectoryManager dirManager(buildRoot, Logger());
 
     // We don't do error checking here, as this is not a convenient place to report an error.
     // If creation of the build directory is not possible, we will get sensible error messages
@@ -308,12 +397,12 @@ static void provideValuesTree(const QVariantMap &values, QVariantMap *valueTree)
 
     valueTree->clear();
     for (QVariantMap::const_iterator it = values.constBegin(); it != values.constEnd(); ++it) {
-        const QString name = it.key();
+        const QString &name = it.key();
         int idx = name.lastIndexOf(QLatin1Char('.'));
         const QStringList nameElements = (idx == -1)
                 ? QStringList() << name
                 : QStringList() << name.left(idx) << name.mid(idx + 1);
-        Internal::setConfigProperty(*valueTree, nameElements, it.value());
+        setConfigProperty(*valueTree, nameElements, it.value());
     }
 }
 
@@ -354,7 +443,7 @@ static QVariantMap expandedBuildConfigurationInternal(const Profile &profile,
         if (err.hasError())
             throw err;
         if (profileKeys.empty())
-            throw ErrorInfo(Internal::Tr::tr("Unknown or empty profile '%1'.").arg(profile.name()));
+            throw ErrorInfo(Tr::tr("Unknown or empty profile '%1'.").arg(profile.name()));
         for (const QString &profileKey : profileKeys) {
             buildConfig.insert(profileKey, profile.value(profileKey, QVariant(), &err));
                 if (err.hasError())
@@ -364,7 +453,7 @@ static QVariantMap expandedBuildConfigurationInternal(const Profile &profile,
 
     // (2) Build configuration name.
     if (configurationName.isEmpty())
-        throw ErrorInfo(Internal::Tr::tr("No build configuration name set."));
+        throw ErrorInfo(Tr::tr("No build configuration name set."));
     buildConfig.insert(QStringLiteral("qbs.configurationName"), configurationName);
     return buildConfig;
 }
@@ -647,4 +736,20 @@ void SetupProjectParameters::setProductErrorMode(ErrorHandlingMode mode)
     d->productErrorMode = mode;
 }
 
+/*!
+ * \brief Indicates how deprecated constructs are handled.
+ */
+DeprecationWarningMode SetupProjectParameters::deprecationWarningMode() const
+{
+    return d->deprecationWarningMode;
+}
+
+/*!
+ * \brief Specifies the behavior on encountering deprecated constructs.
+ */
+void SetupProjectParameters::setDeprecationWarningMode(DeprecationWarningMode mode)
+{
+    d->deprecationWarningMode = mode;
+}
+
 } // namespace qbs
