@@ -1,30 +1,39 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
-** This file is part of the Qt Build Suite.
+** This file is part of Qbs.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms and
-** conditions see http://www.qt.io/terms-conditions. For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
+**
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -51,19 +60,15 @@
 #include "Lexer.h"
 #include <cctype>
 
-using namespace CPlusPlus;
+namespace CPlusPlus {
 
 Lexer::Lexer(const char *firstChar, const char *lastChar)
     : _state(State_Default),
-      _flags(0),
       _currentLine(1)
 {
     setSource(firstChar, lastChar);
 }
 
-Lexer::~Lexer()
-{ }
-
 void Lexer::setSource(const char *firstChar, const char *lastChar)
 {
     _firstChar = firstChar;
@@ -216,7 +221,7 @@ void Lexer::scan_helper(Token *tok)
         goto _Lagain;
 
     case '"': case '\'': {
-        const char quote = ch;
+        const unsigned char quote = ch;
 
         tok->f.kind = quote == '"'
             ? T_STRING_LITERAL
@@ -225,7 +230,7 @@ void Lexer::scan_helper(Token *tok)
         while (_yychar && _yychar != quote) {
             if (_yychar == '\n')
                 break;
-            else if (_yychar != '\\')
+            if (_yychar != '\\')
                 yyinp();
             else {
                 yyinp(); // skip `\\'
@@ -394,7 +399,7 @@ void Lexer::scan_helper(Token *tok)
             bool doxy = false;
 
             if (_yychar == '*' || _yychar == '!') {
-                const char ch = _yychar;
+                const unsigned char ch = _yychar;
 
                 yyinp();
 
@@ -566,7 +571,8 @@ void Lexer::scan_helper(Token *tok)
                 // const int yylen = _currentChar - yytext;
                 //tok->f.kind = classifyObjCAtKeyword(yytext, yylen);		 /// ### FIXME
                 break;
-            } else if (ch == '@' && _yychar == '"') {
+            }
+            if (ch == '@' && _yychar == '"') {
                 // objc @string literals
                 ch = _yychar;
                 yyinp();
@@ -600,7 +606,7 @@ void Lexer::scan_helper(Token *tok)
             ch = _yychar;
             yyinp();
 
-            const char quote = ch;
+            const unsigned char quote = ch;
 
             tok->f.kind = quote == '"'
                 ? T_WIDE_STRING_LITERAL
@@ -659,4 +665,4 @@ void Lexer::scan_helper(Token *tok)
     } // switch
 }
 
-
+} // namespace CPlusPlus
