@@ -285,6 +285,57 @@ void TestBlackboxQt::mocSameFileName()
     QCOMPARE(m_qbsStdout.count("compiling moc_someclass.cpp"), 2);
 }
 
+void TestBlackboxQt::noRelinkOnQDebug()
+{
+    QFETCH(QString, checkMode);
+    QFETCH(bool, expectRelink);
+
+    QVERIFY(QDir::setCurrent(testDataDir + "/no-relink-on-qdebug"));
+    rmDirR("default");
+
+    // Target check.
+    QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    QVERIFY2(m_qbsStdout.contains("is GCC: "), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("is MinGW: "), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("is Darwin: "), m_qbsStdout.constData());
+    const bool isGCCLike = m_qbsStdout.contains("is GCC: true");
+    const bool isMingw = m_qbsStdout.contains("is MinGW: true");
+    const bool isDarwin = m_qbsStdout.contains("is Darwin: true");
+    if (!isGCCLike)
+        expectRelink = false;
+    else if (isMingw || isDarwin)
+        expectRelink = true;
+
+    // Initial build.
+    QbsRunParameters params("resolve");
+    if (isGCCLike && !checkMode.isEmpty())
+        params.arguments << ("modules.cpp.exportedSymbolsCheckMode:" + checkMode);
+    QCOMPARE(runQbs(params), 0);
+    QCOMPARE(runQbs(), 0);
+    QCOMPARE(m_qbsStdout.count("linking"), 2);
+
+    // Inserting the qDebug() statement will pull in weak symbols.
+    WAIT_FOR_NEW_TIMESTAMP();
+    REPLACE_IN_FILE("lib.cpp", "// qDebug", "qDebug");
+    QCOMPARE(runQbs(), 0);
+    QCOMPARE(m_qbsStdout.count("linking"), expectRelink ? 2 : 1);
+
+    // Also check the opposite case.
+    WAIT_FOR_NEW_TIMESTAMP();
+    REPLACE_IN_FILE("lib.cpp", "qDebug", "// qDebug");
+    QCOMPARE(runQbs(), 0);
+    QCOMPARE(m_qbsStdout.count("linking"), expectRelink ? 2 : 1);
+}
+
+void TestBlackboxQt::noRelinkOnQDebug_data()
+{
+    QTest::addColumn<QString>("checkMode");
+    QTest::addColumn<bool>("expectRelink");
+    QTest::newRow("default") << QString() << false;
+    QTest::newRow("relaxed") << QString("ignore-undefined") << false;
+    QTest::newRow("strict") << QString("strict") << true;
+}
+
 void TestBlackboxQt::pkgconfig()
 {
     QDir::setCurrent(testDataDir + "/pkgconfig");
@@ -298,6 +349,54 @@ void TestBlackboxQt::pkgconfig()
         QSKIP("pkgconfig or Qt not found");
 }
 
+void TestBlackboxQt::pkgconfigQt()
+{
+    QFETCH(QStringList, arguments);
+    QFETCH(bool, success);
+
+    QDir::setCurrent(testDataDir + "/pkgconfig-qt");
+    rmDirR(relativeBuildDir());
+
+    QbsRunParameters dumpParams("resolve", {"-f", "dump-libpath.qbs"});
+    QCOMPARE(runQbs(dumpParams), 0);
+    auto lines = QString::fromUtf8(m_qbsStdout).split('\n');
+    const QString needle = "libPath=";
+    qbs::Internal::removeIf(
+            lines, [&needle](const auto &line) { return !line.startsWith(needle); });
+    QCOMPARE(lines.size(), 1);
+    const auto libPath = lines[0].mid(needle.size());
+    auto prefix = QFileInfo(libPath).path();
+    if (prefix.endsWith("/lib") && !prefix.startsWith("/lib"))
+       prefix = QFileInfo(prefix).path();
+    const auto pkgConfigPath = libPath + "/pkgconfig/";
+    if (!QFileInfo(pkgConfigPath).exists())
+        QSKIP("No *.pc files found");
+
+    rmDirR(relativeBuildDir());
+    QbsRunParameters params("build", {"-f", "pkgconfig-qt.qbs"});
+    // need to override prefix for the downloaded Qt
+    params.environment.insert("PKG_CONFIG_QT5CORE_PREFIX", prefix);
+    params.environment.insert("PKG_CONFIG_QT6CORE_PREFIX", prefix);
+    params.arguments << "moduleProviders.qbspkgconfig.extraPaths:" + pkgConfigPath;
+    params.arguments << arguments;
+
+    QCOMPARE(runQbs(params) == 0, success);
+
+    if (!success)
+        QVERIFY(m_qbsStderr.contains("Dependency 'Qt.core' not found for product 'p'"));
+}
+
+void TestBlackboxQt::pkgconfigQt_data()
+{
+    QTest::addColumn<QStringList>("arguments");
+    QTest::addColumn<bool>("success");
+    QTest::newRow("pkgconfig") << QStringList() << true;
+    QTest::newRow("dummy")
+            << QStringList({"products.p.qbsModuleProviders:dummyProvider"}) << false;
+    QTest::newRow("cross-compiling")
+            << QStringList({"moduleProviders.qbspkgconfig.sysroot:/some/fake/sysroot"}) << false;
+}
+
 void TestBlackboxQt::pluginMetaData()
 {
     QDir::setCurrent(testDataDir + "/plugin-meta-data");
@@ -371,6 +470,16 @@ void TestBlackboxQt::pluginSupport()
     }
 }
 
+void TestBlackboxQt::qdoc()
+{
+    QDir::setCurrent(testDataDir + "/qdoc");
+    QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    if (m_qbsStdout.contains("Qt is too old"))
+        QSKIP("Skip test since qdoc3 does not work properly");
+    QCOMPARE(runQbs(), 0);
+    QVERIFY(QFileInfo(relativeProductBuildDir("QDoc Test") + "/qdoctest.qch").exists());
+}
+
 void TestBlackboxQt::qmlDebugging()
 {
     QDir::setCurrent(testDataDir + "/qml-debugging");
@@ -515,10 +624,10 @@ void TestBlackboxQt::staticQtPluginLinking()
     QDir::setCurrent(testDataDir + "/static-qt-plugin-linking");
     QCOMPARE(runQbs(QStringList("products.p.type:application")), 0);
     const bool isStaticQt = m_qbsStdout.contains("Qt is static");
-    QVERIFY2(m_qbsStdout.contains("Creating static import") == isStaticQt, m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("creating static import") == isStaticQt, m_qbsStdout.constData());
     QCOMPARE(runQbs(QbsRunParameters("resolve", QStringList("products.p.type:staticlibrary"))), 0);
     QCOMPARE(runQbs(), 0);
-    QVERIFY2(!m_qbsStdout.contains("Creating static import"), m_qbsStdout.constData());
+    QVERIFY2(!m_qbsStdout.contains("creating static import"), m_qbsStdout.constData());
 }
 
 void TestBlackboxQt::trackAddMocInclude()
